# Java Object-Oriented Programming Design Principles

## Introduction
Java is a versatile and widely-used programming language, rooted in the principles of Object-Oriented Programming (OOP). Understanding these principles is essential for designing efficient, maintainable, and scalable Java applications.

## Markdown Version of the Presentation

---

### 1. Object-Oriented Programming (OOP) Basics

**Object-Oriented Programming** is a programming paradigm based on the concept of "objects," which can contain data and code to manipulate that data. Key features include:

- **Abstraction**  
  Simplifying complex reality by modeling classes based on the essential properties.
  
- **Encapsulation**  
  Bundling the data (attributes) and methods (functions) that operate on the data within one unit, known as a class.
  
- **Inheritance**  
  Mechanism for creating a new class from an existing class, inheriting fields and methods.
  
- **Polymorphism**  
  The ability of different classes to be treated as instances of the same class through a common interface.

---

### 2. Design Principles in Java OOP

Java follows several design principles that foster good software craftsmanship within the realm of Object-Oriented Programming.

#### 2.1. SOLID Principles

**S** - Single Responsibility Principle  
- A class should have one and only one reason to change, meaning that a class should have only one job.

```java
class Book {
    void printText() {
        // printing logic
    }
}

// Better approach using a separate printer class
class BookPrinter {
    void printText(Book book) {
        // printing logic
    }
}
```

**O** - Open/Closed Principle  
- Classes should be open for extension but closed for modification.

```java
abstract class Shape {
    abstract void draw();
}

class Circle extends Shape {
    void draw() {
        // draw circle
    }
}
```

**L** - Liskov Substitution Principle  
- Objects of a superclass should be replaceable with objects of subclasses without affecting the functionality.

```java
class Bird {
    void fly() {
        // flying logic
    }
}

class Ostrich extends Bird {
    @Override
    void fly() {
        throw new UnsupportedOperationException();
    }
}
```

- The solution is not to violate the Liskov principle by ensuring Ostrich is not a subclass of Bird if it doesn't meet all the expectations.

**I** - Interface Segregation Principle  
- Many client-specific interfaces are better than one general-purpose interface.

```java
interface Drivable {
    void drive();
}

interface Flyable {
    void fly();
}

class Drone implements Drivable, Flyable {
    void drive() {
        // driving logic
    }

    void fly() {
        // flying logic
    }
}
```

**D** - Dependency Inversion Principle  
- Rely on abstractions rather than on concrete implementations.

```java
interface Database {
    void connect();
}

class MySQLDatabase implements Database {
    public void connect() {
        // connection logic
    }
}

class Application {
    private Database db;

    public Application(Database db) {
        this.db = db;
    }

    void performDatabaseOperations() {
        db.connect();
    }
}
```

---

### 3. Additional Design Patterns

#### 3.1. Creational Patterns

- **Singleton Pattern**  
  Ensures a class has only one instance and provides a global point to access it.

```java
public class Singleton {
    private static Singleton instance;

    private Singleton() {}

    public static Singleton getInstance() {
        if (instance == null) {
            instance = new Singleton();
        }
        return instance;
    }
}
```

#### 3.2. Structural Patterns

- **Adapter Pattern**  
  Allows incompatible interfaces to work together.

```java
interface EuropeanPlug {
    void connectToEuropeanSocket();
}

class USPluggers {
    void connectToAmericanSocket() {
        // connection logic
    }
}

class PlugAdapter implements EuropeanPlug {
    private USPluggers plugger;

    PlugAdapter(USPluggers plugger) {
        this.plugger = plugger;
    }

    public void connectToEuropeanSocket() {
        plugger.connectToAmericanSocket();
    }
}
```

#### 3.3. Behavioral Patterns

- **Observer Pattern**  
  Defines a one-to-many dependency between objects so that one object changes state, all its dependents are notified.

```java
interface Observer {
    void update();
}

class Subject {
    private List<Observer> observers = new ArrayList<>();

    void attach(Observer observer) {
        observers.add(observer);
    }

    void notifyAllObservers() {
        for (Observer observer : observers) {
            observer.update();
        }
    }
}
```

---

### 4. References

- **Books**
  - "Effective Java" by Joshua Bloch
  - "Design Patterns: Elements of Reusable Object-Oriented Software" by Erich Gamma, Richard Helm, Ralph Johnson, John Vlissides
  
- **Articles**
  - Official Java Documentation: [Oracle Docs](https://docs.oracle.com/en/java/)
  - "SOLID Principles" by Robert C. Martin

- **Online Resources**
  - StackOverflow for community-driven discussions on Java design patterns.
  - Github repositories showcasing Java OOP design principles.

---

By following these design principles, Java developers can create robust, flexible, and reusable software that stands the test of time. Understanding and implementing these principles is a vital part of mastering Java programming.