# Java Control Flow and Object Handling Concepts

## Table of Contents
1. Control Flow Statements
    - If-Else Statements
    - For Loops
    - For-Each Loops
    - While Loops
    - Do-While Loops
    - Switch Statements
    - Switch Statements with Pattern Matching
2. Object and Class Interaction
    - Invoking Methods
    - Accessing Fields
3. Exception Handling
    - Try-Catch Statements
    - Try-With-Resources Statements

---

## 1. Control Flow Statements

### If-Else Statements

#### Definition
If-Else statements control the flow of the program based on boolean conditions. The `if` condition executes the block of code if the condition is true. The `else` block executes if the `if` condition is false.

#### Syntax
```java
if (condition) {
    // code to be executed if condition is true
} else {
    // code to be executed if condition is false
}
```

#### Example
```java
int number = 10;
if (number % 2 == 0) {
    System.out.println("The number is even.");
} else {
    System.out.println("The number is odd.");
}
```

#### References
- "Effective Java" by Joshua Bloch
- Oracle Java Documentation: [If-Else](https://docs.oracle.com/javase/tutorial/java/nutsandbolts/if.html)

### For Loops

#### Definition
For loops iterate over a range of values, commonly used for iterating a fixed number of times.

#### Syntax
```java
for (initialization; condition; increment) {
    // code to be executed
}
```

#### Example
```java
for (int i = 0; i < 5; i++) {
    System.out.println("Number: " + i);
}
```

#### References
- "Java: The Complete Reference" by Herbert Schildt
- Oracle Java Documentation: [For Loop](https://docs.oracle.com/javase/tutorial/java/nutsandbolts/for.html)

### For-Each Loops

#### Definition
The for-each loop is used for traversing elements in collections or arrays.

#### Syntax
```java
for (Type element : collection) {
    // code to be executed
}
```

#### Example
```java
int[] numbers = {1, 2, 3, 4, 5};
for (int number : numbers) {
    System.out.println(number);
}
```

#### References
- "Head First Java" by Kathy Sierra
- Oracle Java Documentation: [Enhanced For Loop](https://docs.oracle.com/javase/1.5.0/docs/guide/language/foreach.html)

### While Loops

#### Definition
While loops execute a block of code repeatedly as long as the given condition is true.

#### Syntax
```java
while (condition) {
    // code to be executed
}
```

#### Example
```java
int i = 0;
while (i < 5) {
    System.out.println("Number: " + i);
    i++;
}
```

#### References
- "Thinking in Java" by Bruce Eckel
- Oracle Java Documentation: [While Loop](https://docs.oracle.com/javase/tutorial/java/nutsandbolts/while.html)

### Do-While Loops

#### Definition
Do-while loops are similar to while loops but execute at least once because the condition is checked after the loop body.

#### Syntax
```java
do {
    // code to be executed
} while (condition);
```

#### Example
```java
int i = 0;
do {
    System.out.println("Number: " + i);
    i++;
} while (i < 5);
```

#### References
- "Java Programming Basics" by Charles E. Leiserson
- Oracle Java Documentation: [Do-While Loop](https://docs.oracle.com/javase/tutorial/java/nutsandbolts/while.html#do)

### Switch Statements

#### Definition
Switch statements allow multi-way branching by comparing the result of an expression against various cases.

#### Syntax
```java
switch (expression) {
    case value1:
        // code to be executed if expression == value1
        break;
    case value2:
        // code to be executed if expression == value2
        break;
    default:
        // code to be executed if expression doesn't match any case
}
```

#### Example
```java
int day = 3;
switch (day) {
    case 1:
        System.out.println("Monday");
        break;
    case 2:
        System.out.println("Tuesday");
        break;
    case 3:
        System.out.println("Wednesday");
        break;
    default:
        System.out.println("Invalid day");
}
```

#### References
- "Core Java Volume I" by Cay S. Horstmann
- Oracle Java Documentation: [Switch Statement](https://docs.oracle.com/javase/tutorial/java/nutsandbolts/switch.html)

### Switch Statements with Pattern Matching

#### Definition
As of Java 17, switch statements can use pattern matching to destruct and bind variables from objects.

#### Example
```java
Object obj = "Hello!";
switch (obj) {
    case Integer i -> System.out.println("Integer " + i);
    case String s -> System.out.println("String " + s);
    default -> System.out.println("Unknown type");
}
```

#### References
- "Java Language Specification"
- Oracle Java Documentation: [Pattern Matching for switch](https://openjdk.java.net/jeps/406)

## 2. Object and Class Interaction

### Invoking Methods

#### Definition
Methods define behaviors of objects. Invoking methods allows object interaction and functionality execution.

#### Invoking Object Methods
```java
class Car {
    void drive() {
        System.out.println("Car is driving");
    }
}

Car myCar = new Car();
myCar.drive();
```

#### Invoking Static (Class) Methods
```java
class MathHelper {
    static int add(int a, int b) {
        return a + b;
    }
}

int sum = MathHelper.add(5, 3);
```

#### References
- "Effective Java" by Joshua Bloch
- "Java: The Complete Reference" by Herbert Schildt

### Accessing Fields

#### Definition
Fields store data for objects. They can be instance fields (belonging to objects) or static fields (belonging to the class).

#### Example
```java
class Dog {
    String name; // instance field
    static int count = 0; // static field

    Dog(String name) {
        this.name = name;
        count++;
    }
}

Dog dog = new Dog("Buddy");
System.out.println(dog.name); // Accessing instance field
System.out.println(Dog.count); // Accessing static field
```

#### References
- "Java: The Complete Reference" by Herbert Schildt
- Oracle Java Documentation: [Classes and Objects](https://docs.oracle.com/javase/tutorial/java/javaOO/classes.html)

## 3. Exception Handling

### Try-Catch Statements

#### Definition
Try-catch blocks handle exceptions, enabling code to recover or gracefully handle errors during execution.

#### Syntax
```java
try {
    // code that may throw exception
} catch (ExceptionType e) {
    // code to handle exception
}
```

#### Example
```java
try {
    int divideByZero = 5 / 0;
} catch (ArithmeticException e) {
    System.out.println("Cannot divide by zero");
}
```

#### References
- "Java Concurrency in Practice" by Brian Goetz
- Oracle Java Documentation: [Exceptions](https://docs.oracle.com/javase/tutorial/essential/exceptions/)

### Try-With-Resources Statements

#### Definition
Introduced in Java 7, try-with-resources simplifies resource management by automatically closing resources.

#### Syntax
```java
try (ResourceType resource = new ResourceType()) {
    // code that uses the resource
}
```

#### Example
```java
try (BufferedReader br = new BufferedReader(new FileReader("file.txt"))) {
    System.out.println(br.readLine());
} catch (IOException e) {
    e.printStackTrace();
}
```

#### References
- "Effective Java" by Joshua Bloch
- Oracle Java Documentation: [Try-With-Resources](https://docs.oracle.com/javase/tutorial/essential/exceptions/tryResourceClose.html)

---

This set of slides serves as an introduction to Java's control flow and object handling, focusing on the practical implementation and conceptual understanding required to write robust Java applications. Make sure to explore the provided references for a deeper understanding of each topic.