## Java I/O

This section covers the basics of Input/Output operations in Java, focusing on reading from and writing to files and streams.

### Introduction

Java I/O uses streams to handle input and output operations.  A stream represents a sequence of data.  There are two main types of streams:

* **Byte Streams:** Used for reading and writing binary data (e.g., images, audio files).  Key classes include `InputStream` and `OutputStream`.
* **Character Streams:** Used for reading and writing text data. Key classes include `Reader` and `Writer`.


### File I/O

#### Reading from a file:

```java
import java.io.*;

public class ReadFile {
    public static void main(String[] args) throws IOException {
        File file = new File("input.txt");
        FileReader fileReader = new FileReader(file);
        BufferedReader bufferedReader = new BufferedReader(fileReader); // Improves efficiency

        String line;
        while ((line = bufferedReader.readLine()) != null) {
            System.out.println(line);
        }

        bufferedReader.close(); // Important: Always close resources
    }
}
```

#### Writing to a file:

```java
import java.io.*;

public class WriteFile {
    public static void main(String[] args) throws IOException {
        File file = new File("output.txt");
        FileWriter fileWriter = new FileWriter(file);
        BufferedWriter bufferedWriter = new BufferedWriter(fileWriter); // Improves efficiency

        bufferedWriter.write("This is the first line.\n");
        bufferedWriter.write("This is the second line.");

        bufferedWriter.close(); // Important: Always close resources
    }
}
```

### Handling Exceptions

I/O operations can throw `IOException`.  Always handle these exceptions using `try-catch` blocks:

```java
try {
    // I/O operation here
} catch (IOException e) {
    System.err.println("An I/O error occurred: " + e.getMessage());
}
```


### References:

* **Book:** Core Java Volume I – Fundamentals (Cay S. Horstmann)
* **Book:** Effective Java (Joshua Bloch)
* **Documentation:** [Java I/O API](https://docs.oracle.com/en/java/javase/17/docs/api/java.base/java/io/package-summary.html)
* **Article:** [Java I/O Tutorial](https://www.baeldung.com/java-io-tutorial) (Baeldung - Replace with actual article link if necessary)



## I/O Decorator Pattern in Java

The Decorator pattern allows you to dynamically add responsibilities to objects without altering their structure. In Java I/O, it's used to enhance streams with additional functionalities like buffering, data conversion, and object serialization.


### How it Works

The decorator pattern involves wrapping a core stream object with a series of decorator objects. Each decorator adds a specific feature.  This builds a chain of decorators, where the outermost decorator handles the final output or input, and each inner decorator successively modifies the stream before passing it to the next decorator in the chain.


### Example: Buffered I/O

`BufferedReader` and `BufferedWriter` are examples of decorators. They add buffering functionality to underlying `FileReader` and `FileWriter` objects:

```java
import java.io.*;

public class BufferedIOExample {

    public static void main(String[] args) throws IOException {
        // Core reader/writer
        FileReader fileReader = new FileReader("input.txt");
        FileWriter fileWriter = new FileWriter("output.txt");

        // Decorators
        BufferedReader bufferedReader = new BufferedReader(fileReader);
        BufferedWriter bufferedWriter = new BufferedWriter(fileWriter);

        String line;
        while ((line = bufferedReader.readLine()) != null) {
            bufferedWriter.write(line + "\n");  // Writing to buffered writer
        }


        bufferedReader.close();
        bufferedWriter.close(); // Close the outermost decorators - underlying streams will also be closed.
    }
}
```


### Other Decorator Examples

* `DataInputStream` and `DataOutputStream`: Add functionality to read and write primitive data types.
* `ObjectInputStream` and `ObjectOutputStream`:  Enable serialization and deserialization of objects.


### Benefits of Decorator Pattern

* **Flexibility:** Add functionalities dynamically without subclassing.
* **Maintainability:**  Code is cleaner and easier to understand.
* **Reusability:** Decorators can be combined in various ways.


### References

* **Book:** Head First Design Patterns (Elisabeth Freeman, Kathy Sierra, Bert Bates)
* **Book:** Design Patterns: Elements of Reusable Object-Oriented Software (Erich Gamma, Richard Helm, Ralph Johnson, John Vlissides) —  "Gang of Four"
* **Documentation:** [Java I/O API](https://docs.oracle.com/en/java/javase/17/docs/api/java.base/java/io/package-summary.html)
* **Article:** [Decorator Design Pattern in Java](https://www.geeksforgeeks.org/decorator-pattern-in-java-with-example/) (GeeksforGeeks - Replace with actual article link if preferred)



This material provides a foundational understanding of Java I/O and the Decorator pattern's application in I/O operations.  Encourage students to explore the provided resources and experiment with different combinations of decorators to gain a deeper understanding of the topic.  Practical exercises involving reading and writing different file types (text, binary, objects) will reinforce their learning.
