## NIO.2 in Java: An Introduction and Deep Dive

This course material covers Java NIO.2 (New Input/Output), a powerful API introduced in Java 7, offering a significant upgrade over the original NIO. NIO.2 simplifies file system operations, provides asynchronous I/O capabilities, and introduces several new features for improved performance and flexibility.

### 1. Introduction to NIO.2

NIO.2 addresses limitations of the original NIO and introduces a more modern file I/O API centered around the `java.nio.file` package. Key features include:

* **`Path` Interface:** Represents a file or directory path, replacing `File` objects.
* **`Files` Class:** Provides utility methods for file operations like copying, moving, deleting, etc.
* **`FileSystem` and `FileSystems`:** Abstraction for interacting with different file systems (local, network, zip, etc.).
* **Asynchronous I/O:**  Allows non-blocking file operations for enhanced performance.
* **`WatchService` API:** Enables monitoring file system changes.


### 2. Working with Paths and Files

The `Path` interface is fundamental to NIO.2.  It provides a platform-independent way to represent files and directories.

```java
import java.nio.file.Path;
import java.nio.file.Paths;

public class PathExample {
    public static void main(String[] args) {
        Path path = Paths.get("my_directory", "my_file.txt"); // Creating a Path
        System.out.println("File name: " + path.getFileName());
        System.out.println("Parent directory: " + path.getParent());
        System.out.println("Absolute path: " + path.toAbsolutePath());
    }
}
```

The `Files` class offers a rich set of utility methods:

```java
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;


public class FilesExample {
    public static void main(String[] args) throws IOException {
        Path source = Paths.get("source.txt");
        Path destination = Paths.get("destination.txt");

        Files.copy(source, destination, StandardCopyOption.REPLACE_EXISTING); // Copying a file
        Files.deleteIfExists(source); // Deleting a file
        Files.createDirectories(Paths.get("new_directory")); // Creating directories

        boolean exists = Files.exists(destination);  // Checking file existence
        System.out.println("File exists: "+ exists);
    }
}
```


### 3. Asynchronous I/O

NIO.2 supports asynchronous I/O through the `AsynchronousFileChannel` class, allowing non-blocking file operations.

```java
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.AsynchronousFileChannel;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.util.concurrent.Future;

public class AsyncIOExample {
    public static void main(String[] args) throws IOException, InterruptedException, java.util.concurrent.ExecutionException {
        Path path = Paths.get("async.txt");

        AsynchronousFileChannel fileChannel = AsynchronousFileChannel.open(path, StandardOpenOption.READ);

        ByteBuffer buffer = ByteBuffer.allocate(1024);
        Future<Integer> readResult = fileChannel.read(buffer, 0); // Asynchronous read

        while (!readResult.isDone()) {
            // Perform other operations while waiting
            System.out.println("Doing other work...");
            Thread.sleep(100); 
        }

        int bytesRead = readResult.get(); // Get the result when ready
        System.out.println("Bytes read: " + bytesRead);
        fileChannel.close();
    }
}
```



### 4.  File System Monitoring with WatchService

`WatchService` allows monitoring file system changes (creation, modification, deletion) in a specified directory.


```java
//  Example using WatchService (Simplified –  error handling and complete implementation omitted for brevity)
import java.nio.file.*;
import static java.nio.file.StandardWatchEventKinds.*;

public class WatchServiceExample {
    public static void main(String[] args) throws IOException, InterruptedException {
        WatchService watcher = FileSystems.getDefault().newWatchService();
        Path dir = Paths.get("."); 
        dir.register(watcher, ENTRY_CREATE, ENTRY_DELETE, ENTRY_MODIFY);

        for (;;) { 
            WatchKey key = watcher.take();  // Blocks until an event is available
            for (WatchEvent<?> event : key.pollEvents()) {
                WatchEvent.Kind<?> kind = event.kind();
                System.out.println("Event kind:" + kind + ". File affected: " + event.context());
            }
            key.reset(); // Reset the key to receive further watch events
         }
    }
}
```


### 5. Further Exploration


* **File Attributes:** NIO.2 provides access to file attributes (e.g., size, last modified time).
* **Symbolic Links:**  Support for working with symbolic links is included.
* **File Visitors:** The `Files.walkFileTree()` method provides a powerful way to traverse file trees.


### References

* **Documentation:** [Java NIO.2 Documentation](https://docs.oracle.com/javase/tutorial/essential/io/fileio.html)
* **Book:**  "Java NIO" by Ron Hitchens (O'Reilly) – Covers both original NIO and NIO.2.
* **Article:** [Baeldung - Java NIO.2](https://www.baeldung.com/java-nio-2-file-api) -  Good overview and examples.
* **Article:** [Oracle -  Trail: Basic I/O (The Java™ Tutorials > Essential Classes)](https://docs.oracle.com/javase/tutorial/essential/io/index.html) -  Comprehensive Java I/O tutorial covering NIO.2.



This course material provides a foundational understanding of NIO.2.  Experimenting with the code examples and referring to the provided resources will further enhance your understanding and allow you to leverage the power of NIO.2 in your Java applications. Remember to handle exceptions appropriately in real-world applications and explore the more advanced features of NIO.2 as needed.