## Dynamic Proxies in Java

This lecture explores dynamic proxies, a powerful mechanism in Java that allows you to create proxy objects at runtime. We'll delve into their purpose, implementation, and practical applications.

**1. Introduction: What are Dynamic Proxies?**

A dynamic proxy acts as a surrogate or placeholder for a real object.  It intercepts method calls to the real object, allowing you to perform actions before, after, or around the actual method execution. This interception provides opportunities for:

* **Logging:** Track method calls, arguments, and return values.
* **Security:** Enforce access control policies before allowing method execution.
* **Transaction Management:** Wrap method calls within transactions.
* **Lazy Loading:** Delay object initialization until necessary.
* **Remote Method Invocation:** Forward method calls to remote objects.

Dynamic proxies are different from static proxies, which are created at compile time. The power of dynamic proxies lies in their ability to be generated on-the-fly, without needing prior knowledge of the target object's interface.

**2. Key Components:**

* **`java.lang.reflect.InvocationHandler` Interface:** This interface defines a single method, `invoke(Object proxy, Method method, Object[] args)`.  The `invoke` method is called whenever a method is invoked on the proxy object.  Your custom invocation handler logic resides here.
* **`java.lang.reflect.Proxy` Class:** This class provides static methods to create dynamic proxy instances.  The `newProxyInstance` method is crucial for creating proxies.


**3. Code Example: Logging Method Calls**

Let's create a dynamic proxy that logs method calls:

```java
import java.lang.reflect.*;

interface MyInterface {
    void doSomething(String arg);
    int calculate(int a, int b);
}

class RealObject implements MyInterface {
    @Override
    public void doSomething(String arg) {
        System.out.println("Real object doing something: " + arg);
    }

    @Override
    public int calculate(int a, int b) {
        return a + b;
    }
}

class LoggingInvocationHandler implements InvocationHandler {
    private final Object target;

    public LoggingInvocationHandler(Object target) {
        this.target = target;
    }

    @Override
    public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
        System.out.println("Calling method: " + method.getName() + " with arguments: " + Arrays.toString(args));
        Object result = method.invoke(target, args);
        System.out.println("Method " + method.getName() + " returned: " + result);
        return result;
    }
}

public class DynamicProxyExample {
    public static void main(String[] args) {
        MyInterface realObject = new RealObject();
        InvocationHandler handler = new LoggingInvocationHandler(realObject);

        MyInterface proxy = (MyInterface) Proxy.newProxyInstance(
                realObject.getClass().getClassLoader(),
                realObject.getClass().getInterfaces(),
                handler);

        proxy.doSomething("Hello");
        int sum = proxy.calculate(5, 3);
        System.out.println("Sum: " + sum);
    }
}

```

**4. Explanation:**

* We define an interface `MyInterface` and a `RealObject` that implements it.
* The `LoggingInvocationHandler` implements `InvocationHandler` and intercepts method calls. It logs the method name, arguments, and return value.
* `Proxy.newProxyInstance` creates the dynamic proxy.  It takes the class loader, the interfaces implemented by the target object, and the invocation handler as arguments.

**5. Practical Applications:**

* **AOP (Aspect-Oriented Programming) Frameworks:** Spring AOP utilizes dynamic proxies extensively to implement aspects like transaction management, security, and logging.
* **ORM (Object-Relational Mapping) Libraries:** Hibernate uses proxies for lazy loading of related entities.
* **Testing Frameworks:** Mockito uses proxies to create mock objects for testing.


**6.  References:**

* **Book:**  *Effective Java* by Joshua Bloch (Item 30: Favor the use of enums over int constants) - Discusses the benefits of using interfaces and relates to the principles used in dynamic proxies.
* **Documentation:** [Java Reflection API](https://docs.oracle.com/en/java/javase/17/docs/api/java.base/java/lang/reflect/package-summary.html) - Official documentation for the `java.lang.reflect` package.
* **Article:**  Baeldung - [Introduction to Dynamic Proxies in Java](https://www.baeldung.com/java-dynamic-proxies) - A comprehensive guide to dynamic proxies.
* **Article:** Oracle - [What is a dynamic proxy?](https://docs.oracle.com/javase/7/docs/technotes/guides/reflection/proxy.html) - Oracle's documentation explaining dynamic proxies.


**7. Exercises:**

* Modify the example to implement a dynamic proxy that performs access control checks before allowing method execution.
* Create a dynamic proxy that measures the execution time of a method.
* Explore the use of CGLIB proxies, which can create proxies for classes that don't implement interfaces.



This material provides a foundation for understanding and utilizing dynamic proxies in Java.  Experimenting with the code and exploring the provided resources will further solidify your understanding of this powerful technique.
