# Java Versions and Their Features

## Introduction

Java is a high-level, class-based, object-oriented programming language that has evolved significantly since its inception in 1995. Each new version of Java has brought enhancements, new features, and performance improvements. This presentation provides a comprehensive overview of the major Java versions, highlighting the key features introduced in each release. Code examples are included to illustrate some of these features.

---

## Java Version History

### Java SE 1.0 (1996)

**Release Date**: January 23, 1996

#### Key Features

- **Initial Release**: The first stable version of Java.
- **Basic Language Features**: Classes, objects, inheritance, exception handling, threads, and the core API.

#### Code Example

```java
public class HelloWorld {
    public static void main(String[] args) {
        System.out.println("Hello, World!");
    }
}
```

---

### Java SE 1.1 (1997)

**Release Date**: February 19, 1997

#### Key Features

- **Inner Classes**: Support for inner classes and nested classes.
- **JavaBeans**: Introduction of JavaBeans component architecture.
- **JDBC**: Java Database Connectivity for database access.
- **RMI**: Remote Method Invocation for distributed computing.

#### Code Example: Inner Classes

```java
public class OuterClass {
    class InnerClass {
        void display() {
            System.out.println("Inner Class Method");
        }
    }
}
```

---

### Java SE 1.2 (1998) - "Java 2"

**Release Date**: December 8, 1998

#### Key Features

- **Collections Framework**: Introduction of the Collections API.
- **JIT Compiler**: Just-In-Time compiler for performance improvements.
- **Swing API**: New GUI components in the Swing library.
- **Java Plug-in**: Integration with web browsers.

#### Code Example: Collections

```java
import java.util.ArrayList;
import java.util.List;

public class CollectionsExample {
    public static void main(String[] args) {
        List<String> list = new ArrayList<>();
        list.add("Java");
        list.add("Python");
        list.forEach(System.out::println);
    }
}
```

---

### Java SE 1.3 (2000)

**Release Date**: May 8, 2000

#### Key Features

- **HotSpot JVM**: Introduction of the HotSpot virtual machine for better performance.
- **Improved RMI**: Performance enhancements in RMI.
- **JavaSound**: New API for audio capabilities.

---

### Java SE 1.4 (2002)

**Release Date**: February 6, 2002

#### Key Features

- **Assertions**: New assertion facility.
- **Exception Chaining**: Enhanced exception handling.
- **Regular Expressions**: Support through `java.util.regex` package.
- **NIO (New I/O)**: Non-blocking I/O operations.

#### Code Example: Assertions

```java
public class AssertionExample {
    public static void main(String[] args) {
        int x = -1;
        assert x >= 0 : "Value of x should be non-negative";
    }
}
```

---

### Java SE 5.0 (2004)

**Release Date**: September 30, 2004

#### Key Features

- **Generics**: Type-safe collections and classes.
- **Enhanced for Loop**: Simplified iteration over collections.
- **Autoboxing/Unboxing**: Automatic conversion between primitives and wrapper classes.
- **Annotations**: Metadata annotations added.
- **Enumerations**: Type-safe enums.

#### Code Example: Generics and Enhanced For Loop

```java
import java.util.ArrayList;
import java.util.List;

public class GenericsExample {
    public static void main(String[] args) {
        List<String> list = new ArrayList<>();
        list.add("Java");
        list.add("C++");

        for (String language : list) {
            System.out.println(language);
        }
    }
}
```

---

### Java SE 6 (2006)

**Release Date**: December 11, 2006

#### Key Features

- **Scripting Language Support**: Integration with scripting languages (JSR 223).
- **Improved Web Services**: Better support for web services.
- **JDBC 4.0**: Enhancements in database connectivity.

---

### Java SE 7 (2011)

**Release Date**: July 28, 2011

#### Key Features

- **Diamond Operator**: Simplify generics instantiation.
- **Try-With-Resources**: Automatic resource management.
- **Strings in Switch**: Switch statements now support `String`.
- **Binary Literals**: Support for binary number literals.
- **Fork/Join Framework**: Simplify parallel programming.

#### Code Example: Try-With-Resources

```java
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

public class TryWithResourcesExample {
    public static void main(String[] args) {
        try (BufferedReader br = new BufferedReader(new FileReader("file.txt"))) {
            System.out.println(br.readLine());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
```

---

### Java SE 8 (2014)

**Release Date**: March 18, 2014

#### Key Features

- **Lambda Expressions**: Introduce functional programming features.
- **Stream API**: Process collections of objects.
- **Date and Time API**: New `java.time` package.
- **Default Methods**: Interfaces can have default implementations.
- **Optional Class**: Handle null values gracefully.

#### Code Example: Lambda Expressions and Stream API

```java
import java.util.Arrays;
import java.util.List;

public class LambdaExample {
    public static void main(String[] args) {
        List<String> list = Arrays.asList("Java", "Python", "C++");
        list.stream()
            .filter(s -> s.startsWith("J"))
            .forEach(System.out::println);
    }
}
```

---

### Java SE 9 (2017)

**Release Date**: September 21, 2017

#### Key Features

- **Module System (Project Jigsaw)**: Modularize the JDK.
- **JShell**: Interactive Java REPL.
- **Private Interface Methods**: Interfaces can have private methods.
- **Factory Methods for Collections**: Easy ways to create immutable collections.

#### Code Example: Creating Immutable List

```java
import java.util.List;

public class FactoryMethodsExample {
    public static void main(String[] args) {
        List<String> list = List.of("Java", "Python", "C++");
        System.out.println(list);
    }
}
```

---

### Java SE 10 (2018)

**Release Date**: March 20, 2018

#### Key Features

- **Local Variable Type Inference**: Use `var` keyword for local variables.
- **Garbage Collector Enhancements**: Improved memory management.

#### Code Example: Local Variable Type Inference

```java
public class VarExample {
    public static void main(String[] args) {
        var message = "Hello, Java 10!";
        System.out.println(message);
    }
}
```

---

### Java SE 11 (2018)

**Release Date**: September 25, 2018

#### Key Features

- **Long-Term Support (LTS)**: First LTS release after Java 8.
- **HTTP Client API**: Standardized HTTP client.
- **Nest-Based Access Control**: Simplify access between nested classes.
- **Removal of Java EE and CORBA Modules**: Cleaned up the JDK.

#### Code Example: HTTP Client API

```java
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.net.URI;

public class HttpClientExample {
    public static void main(String[] args) throws Exception {
        var client = HttpClient.newHttpClient();
        var request = HttpRequest.newBuilder()
            .uri(new URI("https://www.example.com"))
            .build();

        var response = client.send(request, HttpResponse.BodyHandlers.ofString());
        System.out.println(response.body());
    }
}
```

---

### Java SE 12 (2019)

**Release Date**: March 19, 2019

#### Key Features

- **Switch Expressions (Preview)**: Enhanced switch statements.
- **JVM Constants API**: New API to model key class-file and run-time artifacts.

#### Code Example: Switch Expressions

```java
public class SwitchExpressionExample {
    public static void main(String[] args) {
        int day = 5;
        String dayType = switch (day) {
            case 6, 7 -> "Weekend";
            default -> "Weekday";
        };
        System.out.println(dayType);
    }
}
```

---

### Java SE 13 (2019)

**Release Date**: September 17, 2019

#### Key Features

- **Text Blocks (Preview)**: Multiline string literals.
- **Switch Expressions (Second Preview)**: Refinements to switch expressions.

#### Code Example: Text Blocks

```java
public class TextBlocksExample {
    public static void main(String[] args) {
        String json = """
            {
                "name": "John",
                "age": 30
            }
            """;
        System.out.println(json);
    }
}
```

---

### Java SE 14 (2020)

**Release Date**: March 17, 2020

#### Key Features

- **Records (Preview)**: Compact syntax for data classes.
- **Pattern Matching for instanceof (Preview)**: Simplify type checks and casts.
- **Helpful NullPointerExceptions**: Improved NPE messages.

#### Code Example: Records

```java
public class RecordExample {
    public static void main(String[] args) {
        record Person(String name, int age) {}
        Person person = new Person("Alice", 25);
        System.out.println(person.name());
    }
}
```

---

### Java SE 15 (2020)

**Release Date**: September 15, 2020

#### Key Features

- **Sealed Classes (Preview)**: Restrict which classes can extend or implement them.
- **Text Blocks**: Text blocks become a standard feature.
- **Hidden Classes**: Support for dynamic languages.

#### Code Example: Sealed Classes

```java
public abstract sealed class Shape permits Circle, Square {}

public final class Circle extends Shape {}

public final class Square extends Shape {}
```

---

### Java SE 16 (2021)

**Release Date**: March 16, 2021

#### Key Features

- **Records**: Records become a standard feature.
- **Pattern Matching for instanceof**: Becomes standard.
- **Strong Encapsulation**: Encapsulate JDK internals by default.
- **Vector API (Incubator)**: Express vector computations.

#### Code Example: Pattern Matching

```java
public class PatternMatchingExample {
    public static void main(String[] args) {
        Object obj = "Hello, World!";
        if (obj instanceof String s) {
            System.out.println(s.toUpperCase());
        }
    }
}
```

---

### Java SE 17 (2021)

**Release Date**: September 14, 2021

#### Key Features

- **Long-Term Support (LTS)**: Next LTS after Java 11.
- **Sealed Classes**: Becomes standard feature.
- **Pattern Matching for switch (Preview)**: Simplify complex switch logic.
- **Foreign Function & Memory API (Incubator)**: Interact with code and data outside the JVM.

#### Code Example: Sealed Interfaces

```java
public sealed interface Vehicle permits Car, Truck {}

public final class Car implements Vehicle {}

public final class Truck implements Vehicle {}
```

---

### Java SE 18 (2022)

**Release Date**: March 22, 2022

#### Key Features

- **Simple Web Server**: Minimal HTTP server via command line.
- **UTF-8 by Default**: UTF-8 is the default character set.
- **Code Snippets in Java API Documentation**: Improved documentation capabilities.

---

### Java SE 19 (2022)

**Release Date**: September 20, 2022

#### Key Features

- **Virtual Threads (Preview)**: Lightweight threads for concurrency.
- **Structured Concurrency (Incubator)**: Simplify multithreaded programming.
- **Record Patterns (Preview)**: Decompose record values.

#### Code Example: Virtual Threads

```java
public class VirtualThreadsExample {
    public static void main(String[] args) throws InterruptedException {
        Thread.startVirtualThread(() -> {
            System.out.println("Hello from a virtual thread");
        }).join();
    }
}
```

---

### Java SE 20 (2023)

**Release Date**: March 21, 2023

#### Key Features

- **Scoped Values (Incubator)**: Share data within and across threads.
- **Record Patterns and Pattern Matching for switch (Second Preview)**: Enhancements to pattern matching.

---

### Java SE 21 (2023)

**Release Date**: September 19, 2023

#### Key Features

- **Long-Term Support (LTS)**: Next LTS release.
- **Virtual Threads**: Becomes a standard feature.
- **Pattern Matching for switch**: Becomes standard.
- **Sequenced Collections**: New interfaces for collections with defined encounter order.
- **String Templates (Preview)**: Simplify embedding expressions in strings.

#### Code Example: String Templates

```java
public class StringTemplatesExample {
    public static void main(String[] args) {
        int x = 10;
        String message = STR."Value of x is \{x}";
        System.out.println(message);
    }
}
```

---

## Conclusion

Java has continuously evolved to meet the changing needs of developers and the industry. Each version has introduced features that improve performance, enhance security, and simplify coding. From the introduction of generics in Java 5 to the addition of virtual threads in Java 19 and beyond, Java remains a robust and versatile language.

---

## References

1. [Java SE Documentation](https://docs.oracle.com/javase/)
2. [Java Version History - Wikipedia](https://en.wikipedia.org/wiki/Java_version_history)
3. [OpenJDK Releases](https://openjdk.org/projects/jdk/)
4. [Oracle Java SE Support Roadmap](https://www.oracle.com/java/technologies/java-se-support-roadmap.html)
5. [JDK Enhancement Proposals (JEPs)](https://openjdk.org/jeps/0)

---

**Note**: The code examples provided are for illustrative purposes and may require appropriate JDK versions to compile and run successfully. Some features are in preview or incubator stages in their respective versions and may not be enabled by default.