## Java Design Principles: Building Robust and Maintainable Software

This lecture delves into the core design principles crucial for developing robust, maintainable, and scalable Java applications. We'll explore each principle with illustrative examples and relevant code snippets, adhering to best practices. 

---
### 1. SOLID Principles

SOLID is an acronym representing five key design principles that promote code clarity, flexibility, and maintainability:

**a) Single Responsibility Principle (SRP):**

* **Definition:** Each class or module should have only one specific responsibility. 
* **Benefits:** Enhances readability, simplifies testing, and reduces the impact of changes.

**Example:**

```java
// Violation of SRP: Class responsible for both data storage and formatting
class Employee {
    // Data attributes...
    public void saveEmployeeData(Employee emp) { /* Implementation */ } 
    public String formatEmployeeReport(Employee emp) { /* Implementation */ }
}

// Adhering to SRP: Separate classes for data storage and formatting
class Employee {
    // Data attributes...
}
class EmployeeDAO {
    public void saveEmployeeData(Employee emp) { /* Implementation */ } 
}
class EmployeeFormatter {
    public String formatEmployeeReport(Employee emp) { /* Implementation */ }
}
```

**b) Open/Closed Principle (OCP):**

* **Definition:** Software entities (classes, modules, etc.) should be open for extension but closed for modification.
* **Benefits:** Enables adding new functionality without altering existing code, reducing regression risks.

**Example:**

```java
// Violation of OCP: Modifying the AreaCalculator when a new shape is introduced
class AreaCalculator {
    public double calculateArea(Rectangle[] shapes) { /* Implementation */ }
    public double calculateArea(Circle[] shapes) { /* Implementation */ } // Modification
}

// Adhering to OCP: Using an interface to extend functionality
interface Shape {
    double calculateArea();
}
class Rectangle implements Shape { /* Implementation */ }
class Circle implements Shape { /* Implementation */ } 
class AreaCalculator {
    public double calculateArea(Shape[] shapes) { /* Implementation using polymorphism */ }
}
```

**c) Liskov Substitution Principle (LSP):**

* **Definition:**  Subtypes should be substitutable for their base types without altering the correctness of the program.
* **Benefits:**  Ensures consistency in inheritance hierarchies, promoting code reusability and reliability.

**Example:**

```java
// Violation of LSP: Square should not inherit from Rectangle due to different behavior
class Rectangle {
    public void setWidth(int width) { /* Implementation */ }
    public void setHeight(int height) { /* Implementation */ }
}
class Square extends Rectangle { 
    @Override 
    public void setWidth(int width) { 
        // Sets both width and height, violating the Rectangle contract
        super.setWidth(width); 
        super.setHeight(width); 
    }
    // Similar implementation for setHeight
}

// Adhering to LSP:  Use composition or separate interfaces for distinct behavior
interface Shape {
    double calculateArea();
}
class Rectangle implements Shape { /* Implementation */ }
class Square implements Shape { /* Implementation */ }
```

**d) Interface Segregation Principle (ISP):**

* **Definition:** Clients should not be forced to depend on methods they don't use. Interfaces should be small and focused.
* **Benefits:** Improves flexibility and reduces coupling between classes.

**Example:**

```java
// Violation of ISP: Single interface with multiple responsibilities
interface Worker {
    void work();
    void eat();
    void sleep();
}

// Adhering to ISP: Separate interfaces for different responsibilities
interface Workable {
    void work();
}
interface Eatable {
    void eat();
}
interface Sleepable {
    void sleep();
}
```

**e) Dependency Inversion Principle (DIP):**

* **Definition:**  High-level modules should not depend on low-level modules. Both should depend on abstractions. Abstractions should not depend on details. Details should depend upon abstractions.
* **Benefits:**  Reduces coupling, promotes reusability, and improves testability.

**Example:**

```java
// Violation of DIP: High-level module directly depends on low-level module
class MySqlConnection { 
    // Low-level implementation details...
}
class EmployeeService { 
    private MySqlConnection connection; 
    public EmployeeService() {
        this.connection = new MySqlConnection(); 
    }
    // Methods using the MySqlConnection
}

// Adhering to DIP: Injecting dependencies through abstractions
interface DatabaseConnection {
    // Methods for database operations
}
class MySqlConnection implements DatabaseConnection { /* Implementation */ }
class EmployeeService {
    private DatabaseConnection connection;
    public EmployeeService(DatabaseConnection connection) { 
        this.connection = connection;
    }
    // Methods using the DatabaseConnection interface 
}
```

**Further Reading:**

* [SOLID Principles of Object Oriented Design](https://www.digitalocean.com/community/conceptual_articles/s-o-l-i-d-the-first-five-principles-of-object-oriented-design)
* [SOLID Principles in Java](https://www.baeldung.com/solid-principles)

---
### 2. DRY (Don't Repeat Yourself)

* **Definition:**  Avoid redundant code by abstracting common functionalities into reusable components.
* **Benefits:** Improves code maintainability, reduces errors, and enhances consistency.

**Example:**

```java
// Violation of DRY: Duplicated validation logic
if (username.isEmpty() || username.length() < 6) {
    // Error handling...
}
// ... similar validation in different parts of the code

// Adhering to DRY: Extracting validation logic into a reusable method
public boolean isValidUsername(String username) {
    return !username.isEmpty() && username.length() >= 6;
} 
```

---
### 3. KISS (Keep It Simple, Stupid)

* **Definition:** Favour simplicity over unnecessary complexity. Choose clear, concise solutions.
* **Benefits:** Makes code easier to understand, maintain, and debug.

**Example:**

```java
// Overly complex and less readable solution
// ...multiple nested loops, conditions, and intricate logic...

// Simpler and more maintainable approach 
// ...using clear control flow, appropriate data structures, and modular design...
```

---
### 4. YAGNI (You Aren't Gonna Need It)

* **Definition:** Avoid adding functionality until it's truly needed. Premature optimization can lead to over-engineering.
* **Benefits:** Prevents wasted effort, keeps the codebase leaner, and allows for easier adaptation to future changes.

**Example:**

```java
//  Implementing complex caching mechanisms before performance needs are determined 

// Focus on core features first, implement caching only if performance bottlenecks arise later
```

---
### 5. Law of Demeter (Principle of Least Knowledge)

* **Definition:**  An object should have limited knowledge of other objects and only interact with objects closely related to it.
* **Benefits:** Reduces coupling and increases the modularity of a system.

**Example:**

```java
// Violation: Accessing data through multiple layers of objects
class Wallet {
    public int getMoneyFromCreditCard(CreditCard card) { 
        return card.getAccount().getBalance();  
    }
}

// Adhering to LoD: Encapsulating the operation within the CreditCard class
class CreditCard {
    public int getAvailableBalance() { 
        // Accessing the Account object internally
    }
}
class Wallet {
    public int getMoneyFromCreditCard(CreditCard card) {
        return card.getAvailableBalance(); 
    }
}
```

---
By applying these core Java design principles diligently, you'll produce high-quality, maintainable code that stands the test of time. Remember, prioritize writing clean, modular code over overly complex solutions to foster a robust and adaptable codebase. 
