# Encapsulation in Java: An Object-Oriented Programming Concept

Welcome to this lecture on Encapsulation in Java, a fundamental concept within Object-Oriented Programming (OOP). In this session, we will explore what encapsulation is, understand its purpose, and see how it can be implemented in Java with practical examples. Let's delve into how encapsulation helps in creating robust and maintainable Java applications.

## Table of Contents
1. [Introduction to Encapsulation](#introduction-to-encapsulation)
2. [Benefits of Encapsulation](#benefits-of-encapsulation)
3. [Implementing Encapsulation in Java](#implementing-encapsulation-in-java)
    - [Example with Code](#example-with-code)
4. [Access Modifiers Explained](#access-modifiers-explained)
5. [Practical Example](#practical-example)
6. [References](#references)

## Introduction to Encapsulation

Encapsulation is one of the four core concepts of Object-Oriented Programming: Encapsulation, Inheritance, Polymorphism, and Abstraction. It can be defined as the mechanism of wrapping the data (variables) and code acting on the data (methods) together as a single unit.

### Key Points
- **Encapsulation conceals the internal state and functionality of an object.**
- It restricts direct access to some components and protects the object's data integrity.
- Through encapsulation, internal representations are hidden from the outside, only allowing access through public methods.

## Benefits of Encapsulation

1. **Data Hiding**: By hiding the variables of a class, you ensure that only methods within the class can access or modify those variables. This reduces system complexity and increases robustness by preventing unintended interference and misuse.

2. **Increased Security**: Controlled access improves software security by limiting how data and actions can be invoked or manipulated.

3. **Flexibility and Maintainability**: Encapsulation allows easy maintenance and improvement of code, as changes to the implementation don't affect other parts of the program that rely on the class interface.

4. **Reusability**: Encapsulated code can be reused without needing to understand the intricate details of implementation.

## Implementing Encapsulation in Java

To implement encapsulation in Java, follow these steps:

1. **Declare Class Variables as Private**: This restricts direct access from outside the class.
2. **Provide Public Getter and Setter Methods**: These allow controlled access and modification.

### Example with Code

```java
public class Student {
    // private variables
    private String name;
    private int age;

    // getter method for name
    public String getName() {
        return name;
    }

    // setter method for name
    public void setName(String newName) {
        name = newName;
    }

    // getter method for age
    public int getAge() {
        return age;
    }

    // setter method for age
    public void setAge(int newAge) {
        age = newAge;
    }
}
```

### Explanation
- **Private Variables**: The `name` and `age` variables are private to `Student` class, which means they can't be accessed directly from outside the class.
- **Public Methods**: Public `get` and `set` methods allow controlled access to the private variables: `name` and `age`.

## Access Modifiers Explained

Java uses access modifiers to set the access level for classes, variables, constructors, and methods. Understanding these is key to implementing encapsulation:
- **private**: Accessible only within the class.
- **default**: Accessible within the same package.
- **protected**: Accessible within the same package and subclasses.
- **public**: Accessible from any other class.

## Practical Example

Let's create a class `BankAccount` to illustrate a practical application of encapsulation:

```java
public class BankAccount {
    // private variables
    private double balance;
    private String accountNumber;

    // constructor
    public BankAccount(String accountNumber) {
        this.accountNumber = accountNumber;
        this.balance = 0.0;
    }

    // method to deposit funds
    public void deposit(double amount) {
        if (amount > 0) {
            balance += amount;
        }
    }

    // method to withdraw funds
    public boolean withdraw(double amount) {
        if (amount > 0 && amount <= balance) {
            balance -= amount;
            return true;
        } else {
            return false;
        }
    }

    // getter method for balance
    public double getBalance() {
        return balance;
    }

    // getter method for account number
    public String getAccountNumber() {
        return accountNumber;
    }
}
```

### Explanation
- **Private Data**: The `balance` and `accountNumber` fields are private, ensuring that they cannot be modified directly from outside the `BankAccount` class.
- **Controlled Access**: Methods `deposit`, `withdraw`, and `getBalance` provide a controlled way to modify and access the data.

## References

To deepen your understanding of encapsulation and other OOP principles in Java, consider exploring the following resources:

1. **Books**:
   - "Java: The Complete Reference" by Herbert Schildt
   - "Effective Java" by Joshua Bloch
   - "Thinking in Java" by Bruce Eckel

2. **Online Resources**:
   - [Oracle Java Documentation](https://docs.oracle.com/javase/tutorial/java/concepts/)
   - [The Java™ Tutorials](https://docs.oracle.com/javase/tutorial/)

3. **Articles**:
   - "Encapsulation in Java" on [GeeksforGeeks](https://www.geeksforgeeks.org/encapsulation-in-java/)
   - "Understanding Encapsulation, Inheritance, and Polymorphism" on [JavaWorld](https://www.javaworld.com/article/2075459/)

By understanding and applying encapsulation, you will be well on your way to writing well-structured and efficient Java programs. Thank you for attending this lecture, and I look forward to your questions and discussions.