# Generics in Java

## Slide 1: Introduction to Generics

**Title:**  Unlocking Type Safety with Generics in Java

**Content:**

Generics, introduced in Java 5, are a powerful feature that allows you to write type-safe code by parameterizing types. Before generics, collections and other data structures stored objects as raw types, leading to potential `ClassCastException` errors at runtime. Generics provide compile-time type checking, enhancing code readability, maintainability, and safety.

**Example (Pre-Generics):**

```java
List list = new ArrayList();
list.add("Hello");
list.add(123); // No compile-time error

String str = (String) list.get(1); // Runtime ClassCastException
```

**Example (With Generics):**

```java
List<String> list = new ArrayList<>();
list.add("Hello");
// list.add(123); // Compile-time error: incompatible types
String str = list.get(0); // No casting needed
```

## Slide 2: Generic Classes and Interfaces

**Title:** Defining Generic Types

**Content:**

You can define generic classes, interfaces, and methods by using type parameters enclosed in angle brackets `<>`. These type parameters act as placeholders for concrete types that will be specified when the generic type is used.

**Example: Generic Class**

```java
public class Box<T> {
    private T value;

    public Box(T value) {
        this.value = value;
    }

    public T getValue() {
        return value;
    }
}

// Usage:
Box<String> stringBox = new Box<>("Hello");
String str = stringBox.getValue();

Box<Integer> intBox = new Box<>(123);
Integer num = intBox.getValue();
```

**Example: Generic Interface**

```java
interface Pair<K, V> {
    K getKey();
    V getValue();
}
```


## Slide 3: Type Parameters and Wildcards

**Title:** Working with Type Parameters and Wildcards

**Content:**

* **Type Parameters:**  Represent specific types within a generic class or method.
* **Wildcards:** Provide flexibility when dealing with unknown types.  `?` represents an unknown type.


* **Upper Bounded Wildcard:** `<? extends T>`  -  Accepts types that are `T` or its subtypes. Useful for reading from a generic type.
* **Lower Bounded Wildcard:** `<? super T>` - Accepts types that are `T` or its supertypes.  Useful for writing to a generic type.
* **Unbounded Wildcard:** `<?>` - Accepts any type.


**Example: Wildcards**

```java
public void printList(List<?> list) {
    for (Object obj : list) {
        System.out.println(obj);
    }
}

List<Integer> intList = Arrays.asList(1, 2, 3);
printList(intList); // Works with any type of list


List<? extends Number> numList = new ArrayList<Integer>(); // Integer is a subtype of Number
// numList.add(123); // Compile-time error: can't add to an upper bounded wildcard list

List<? super Integer> intList2 = new ArrayList<Number>(); // Number is a supertype of Integer
intList2.add(123); // Allowed: can add Integer or its subtypes to a lower bounded wildcard list

```

## Slide 4: Generic Methods

**Title:** Writing Flexible Methods with Generics

**Content:**

Generic methods can be defined independently of generic classes.  Type parameters are declared before the return type.

**Example:**

```java
public static <T> T max(T x, T y) {
    return (x.compareTo(y) > 0) ? x : y;
}

// Usage
String maxString = max("apple", "banana");
Integer maxInt = max(10, 20);
```



## Slide 5: Type Erasure

**Title:** Understanding Type Erasure

**Content:**

At runtime, generic type information is erased by the compiler. This is done for backward compatibility with older versions of Java. This means that `List<String>` and `List<Integer>` become simply `List` at runtime.


## Slide 6: Further Reading and Resources

**Content:**

* **The Java Tutorials - Generics:** [https://docs.oracle.com/javase/tutorial/java/generics/index.html](https://docs.oracle.com/javase/tutorial/java/generics/index.html)
* **Effective Java (3rd Edition) by Joshua Bloch:**  Covers generics in detail, including best practices and common pitfalls.
* **Java Generics and Collections by Maurice Naftalin and Philip Wadler:**  A comprehensive guide to generics and how they interact with collections.



This Markdown presentation provides a comprehensive overview of generics in Java, including code examples and links to relevant resources.  Remember to adapt and expand on this material based on your specific course requirements and student level. You can add more examples, quizzes, and interactive exercises to make the learning experience more engaging.