## Generic Classes in Java

**Slide 1: Introduction**

* **Title:** Unleashing the Power of Generics in Java
* **Content:**
    * Generics, introduced in Java 5, are a powerful feature that allows you to write type-safe and reusable code by parameterizing types.
    * Before generics, collections and other data structures worked with the `Object` type, leading to potential `ClassCastException` errors at runtime.
    * Generics address this by enabling compile-time type checking, enhancing code clarity and maintainability.

**Slide 2: What are Generic Classes?**

* **Title:** Understanding Generic Classes
* **Content:**
    * A generic class is defined with a type parameter (e.g., `<T>`, `<E>`, `<K>`, `<V>` - convention suggests using single uppercase letters), which acts as a placeholder for a specific type.
    * When you instantiate a generic class, you provide the concrete type argument.
    * Example: `List<String>` -  `List` is the generic class, `String` is the type argument.  This creates a list that can only hold `String` objects.

**Slide 3: Syntax and Example**

* **Title:** Defining and Using Generic Classes
* **Content:**
```java
// Generic class definition
public class Box<T> {
    private T value;

    public Box(T value) {
        this.value = value;
    }

    public T getValue() {
        return value;
    }
}

// Usage
public static void main(String[] args) {
    Box<Integer> intBox = new Box<>(10);  // Integer box
    int intValue = intBox.getValue();      // No casting needed

    Box<String> stringBox = new Box<>("Hello"); // String box
    String stringValue = stringBox.getValue(); // No casting needed
}
```

**Slide 4: Benefits of Generics**

* **Title:** Why Use Generics?
* **Content:**
    * **Type Safety:** Compile-time type checking prevents runtime `ClassCastException` errors.
    * **Code Reusability:** Write code once and use it with different types.
    * **Code Readability:**  Clearer code as type information is explicitly stated.
    * **Elimination of Casts:** No need for explicit type casting.

**Slide 5: Multiple Type Parameters**

* **Title:** Working with Multiple Type Parameters
* **Content:**
    * Generic classes can have multiple type parameters.
* **Code Example:**
```java
public class Pair<K, V> {
    private K key;
    private V value;

    public Pair(K key, V value) {
        this.key = key;
        this.value = value;
    }

    // ... getters for key and value
}

// Usage
Pair<String, Integer> pair = new Pair<>("name", 25);
```

**Slide 6: Bounded Type Parameters**

* **Title:** Restricting Type Arguments
* **Content:**
    *  Use bounded type parameters to restrict the types that can be used as arguments.
    *  `extends` keyword specifies an upper bound.
* **Code Example:**
```java
public class NumberBox<T extends Number> { // Only Number or its subtypes
    private T value;
    // ...
}

NumberBox<Integer> intBox = new NumberBox<>(10); // Valid
NumberBox<String> stringBox = new NumberBox<>("ten"); // Compile-time error
```

**Slide 7: Wildcards**

* **Title:**  Flexible Type Arguments with Wildcards
* **Content:**
    *  Wildcards (`?`) allow you to work with unknown types in a flexible way.
    *  `? extends T`:  Upper bounded wildcard (can read, but not write).
    *  `? super T`:  Lower bounded wildcard (can write, but reading is restricted).
* **Code Example:**
```java
List<? extends Number> numberList = new ArrayList<Integer>(); // Can hold Integer or its subtypes
// numberList.add(10); // Compile-time error (can't add)
Number num = numberList.get(0); // Can read as Number
```

**Slide 8:  Generic Methods**

* **Title:** Type Parameters in Methods
* **Content:**
    * Methods can also have type parameters, independent of the class's type parameters.

* **Code Example:**

```java
public class Utility {
  public static <T> void printArray(T[] array) {
      for (T element : array) {
          System.out.print(element + " ");
      }
      System.out.println();
  }
}

Integer[] intArray = {1, 2, 3};
String[] stringArray = {"a", "b", "c"};

Utility.printArray(intArray);
Utility.printArray(stringArray);
```


**Slide 9: References**

* **Title:** Further Reading
* **Content:**
    * **The Java Tutorials - Generics:** [https://docs.oracle.com/javase/tutorial/java/generics/index.html](https://docs.oracle.com/javase/tutorial/java/generics/index.html)
    * **Effective Java (3rd Edition) by Joshua Bloch:**  (Covers generics in detail – highly recommended).
    * **Java Generics and Collections by Maurice Naftalin and Philip Wadler:** (A more in-depth exploration of generics).



This detailed Markdown presentation provides a comprehensive overview of generic classes in Java, suitable for an academic teaching environment.  It covers key concepts, syntax, benefits, and advanced topics like bounded type parameters and wildcards, supported by clear code examples and links to valuable resources. Remember to adapt the content and pacing to your specific audience and learning objectives.  You can enhance this presentation further by including interactive exercises, quizzes, and real-world examples.