# Bounded Type Parameters in Java

## Introduction

Generics, introduced in Java 5, allow us to write type-safe and reusable code by parameterizing types. Bounded type parameters further enhance this by restricting the types that can be used as type arguments, ensuring stronger type checking and enabling more specific operations within generic methods or classes.

## What are Bounded Type Parameters?

Bounded type parameters place constraints on the type arguments that can be passed to a generic type.  This is achieved using the `extends` and `super` keywords.

* **Upper Bound:** `extends` keyword specifies an upper bound.  This means the type argument must be a subtype of the specified type (or the type itself).
* **Lower Bound:** `super` keyword specifies a lower bound. This means the type argument must be a supertype of the specified type (or the type itself).


## Upper Bounds with `extends`

This is the most common type of bounded type parameter.  It restricts the type argument to be a subtype of a specific class or interface.  This allows us to invoke methods defined in the upper bound type within the generic code.

```java
public static <T extends Number> double sum(List<T> list) {
    double sum = 0.0;
    for (T num : list) {
        sum += num.doubleValue(); // Accessing Number's method
    }
    return sum;
}

// Usage:
List<Integer> integers = Arrays.asList(1, 2, 3);
double sumInt = sum(integers); // Valid

List<String> strings = Arrays.asList("a", "b");
// double sumStr = sum(strings); // Compile-time error: String doesn't extend Number


public class Box<T extends Comparable<T>> {  // T must implement Comparable
    private T value;

    public T getValue() {
        return value;
    }

    public int compareTo(Box<T> other) {  // Can use compareTo because of the bound
        return value.compareTo(other.getValue());
    }
}
```

**Benefits of Upper Bounds:**

* **Type safety:** Ensures that the type argument supports specific methods.
* **Code reusability:** Allows writing generic algorithms that operate on a range of types.
* **Improved readability:**  Clearly defines the expected type capabilities.

## Lower Bounds with `super`

Lower bounds are less common but provide flexibility when working with generics, especially when dealing with producer-consumer scenarios. They restrict the type argument to be a supertype of a specific type.

```java
public static <T> void addNumbers(List<? super Integer> list) {
    for (int i = 0; i < 10; i++) {
        list.add(i); // Adding Integer is safe since list accepts Integer's supertypes
    }
}

// Usage:
List<Number> numbers = new ArrayList<>();
addNumbers(numbers); // Valid: Number is a supertype of Integer

List<Integer> integers = new ArrayList<>();
addNumbers(integers); // Valid: Integer is a supertype of itself

List<Double> doubles = new ArrayList<>();
// addNumbers(doubles); // Compile-time error: Double is not a supertype of Integer
```

**Benefits of Lower Bounds:**

* **Flexibility with wildcards:**  Powerful when used with wildcards (`?`).
* **Producer-consumer scenarios:** Allows adding elements of a specific type to a collection of its supertypes.

## Multiple Bounds

A type parameter can have multiple upper bounds, separated by `&`. The first bound can be a class, but subsequent bounds must be interfaces.

```java
public static <T extends Number & Comparable<T>> T findMax(List<T> list) {
    // ... implementation using both Number and Comparable methods ...
}
```

## Wildcards vs. Bounded Type Parameters

Wildcards (`? extends T`, `? super T`) and bounded type parameters offer similar functionalities but are used in different contexts:

* **Bounded type parameters:** Used in method and class declarations to define the constraints on type arguments.
* **Wildcards:** Used in method arguments and return types to increase flexibility when working with generic types. They are particularly useful when you don't need to know the specific type argument but want to ensure type safety.



## Resources

* **Java Tutorials - Generics:** [https://docs.oracle.com/javase/tutorial/java/generics/](https://docs.oracle.com/javase/tutorial/java/generics/)
* **Effective Java (3rd Edition) by Joshua Bloch:** Chapter 5 - Generics
* **Java Generics and Collections by Maurice Naftalin and Philip Wadler:** Provides in-depth coverage of generics.



## Conclusion

Bounded type parameters are essential for writing powerful and type-safe generic code in Java. Understanding the nuances of upper and lower bounds, along with their interaction with wildcards, allows developers to create more flexible and reusable code. This presentation provides a solid foundation for working with bounded type parameters, empowering you to leverage the full potential of Java generics.
