## Java: Embracing the Power of Object-Oriented Programming (OOP)

**Course Material:**

This lecture delves into the core principles of Object-Oriented Programming (OOP) within the Java programming language. We'll explore the "four pillars" of OOP, illustrating their significance with practical examples and code snippets. 

**1. Abstraction:**

* **Concept:** Hiding complex implementation details and exposing only essential information to the user. Imagine a TV remote; you interact with buttons (abstraction) without needing to understand the intricate electronics behind them.
* **Benefits:**
    * **Simplicity:** Makes code easier to understand and manage.
    * **Flexibility:** Allows for modifications without affecting other parts of the program.
    * **Code reusability:** Promotes modular design, enhancing reusability.
* **Example:**

```java
// Abstract class representing a generic vehicle
abstract class Vehicle {
    // Abstract method (implementation hidden)
    public abstract void start();

    // Concrete method (common functionality)
    public void stop() {
        System.out.println("Vehicle stopped.");
    }
}

// Concrete class extending Vehicle and providing implementation for start()
class Car extends Vehicle {
    @Override
    public void start() {
        System.out.println("Car started with a key.");
    }
}
```

* **Reference:** [https://docs.oracle.com/javase/tutorial/java/concepts/abstraction.html](https://docs.oracle.com/javase/tutorial/java/concepts/abstraction.html)


**2. Encapsulation:**

* **Concept:** Bundling data (attributes) and the methods (behavior) that operate on the data within a single unit (class). Think of it like a capsule containing both medicine and its instructions.
* **Benefits:**
    * **Data protection:** Controls access to data and prevents accidental modification.
    * **Code maintainability:** Changes within a class have minimal impact elsewhere.
    * **Increased modularity:** Enhances code organization and reusability.
* **Example:**

```java
class BankAccount {
    // Private data members (encapsulated)
    private String accountNumber;
    private double balance;

    // Public methods to access and modify data
    public double getBalance() {
        return this.balance; 
    }

    public void deposit(double amount) {
        // Logic for deposit
        this.balance += amount;
    }
}
```

* **Reference:** [https://docs.oracle.com/javase/tutorial/java/classes/index.html](https://docs.oracle.com/javase/tutorial/java/classes/index.html)


**3. Inheritance:**

* **Concept:** Creating new classes (child classes) based on existing classes (parent classes), inheriting their properties and behaviors. Imagine a family tree where children inherit traits from their parents.
* **Benefits:**
    * **Code reusability:** Avoids code duplication by inheriting from parent classes.
    * **Extensibility:** Facilitates adding new functionalities without modifying existing code.
    * **Polymorphism:** Enables objects of different classes to be treated as objects of a common type.
* **Example:**

```java
// Parent class
class Animal {
    public void eat() {
        System.out.println("Animal is eating");
    }
}

// Child class inheriting from Animal
class Dog extends Animal {
    public void bark() {
        System.out.println("Woof!");
    }
}
```

* **Reference:** [https://docs.oracle.com/javase/tutorial/java/IandI/subclasses.html](https://docs.oracle.com/javase/tutorial/java/IandI/subclasses.html)


**4. Polymorphism:**

* **Concept:** Allows objects of different classes to be treated as objects of a common type.  The word "polymorphism" itself means "many forms."
* **Types:**
    * **Compile-time Polymorphism (Method Overloading):** Defining multiple methods with the same name but different parameters within the same class.
    * **Runtime Polymorphism (Method Overriding):**  A subclass provides a specific implementation for a method that is already defined in its superclass.
* **Benefits:**
    * **Flexibility:**  Write more generic and reusable code.
    * **Code clarity:** Reduces complexity by handling different object types seamlessly.
* **Example:**

```java
class Shape {
    public void draw() {
        System.out.println("Drawing a shape");
    }
}

class Circle extends Shape {
    @Override
    public void draw() { 
        System.out.println("Drawing a circle"); 
    }
}

class Square extends Shape {
    @Override
    public void draw() {
        System.out.println("Drawing a square");
    }
}

// Polymorphic behavior
public class Main {
    public static void main(String[] args) {
        Shape shape1 = new Circle();
        Shape shape2 = new Square();

        shape1.draw(); // Output: Drawing a circle
        shape2.draw(); // Output: Drawing a square
    }
}
```

* **Reference:** [https://docs.oracle.com/javase/tutorial/java/IandI/polymorphism.html](https://docs.oracle.com/javase/tutorial/java/IandI/polymorphism.html) 


**Conclusion:**

Mastering these OOP principles in Java empowers you to write cleaner, more maintainable, and robust code. By understanding the "why" and "how" of abstraction, encapsulation, inheritance, and polymorphism, you unlock the true potential of object-oriented programming for building complex and scalable applications. 
