## Java Programming: Structure & Naming Conventions

Welcome to this introductory course on Java programming, where we'll delve into the fundamental structure of a Java program and the crucial role of naming conventions in writing clean, readable, and maintainable code. 

---
## Section 1: Structure of a Java Program

A Java program consists of several key elements organized in a specific manner. Understanding this structure is crucial for writing and interpreting Java code. Let's break it down:

1. **Package Declaration (Optional):**
   - Used to group related classes and interfaces.
   - Syntax: `package packageName;`
   - Example: `package com.example.myproject;`
   - Placed at the very beginning of the file.
   - More details in **Section 3: Java Package Naming Conventions**.

2. **Import Statements (Optional):**
   - Used to include classes from other packages.
   - Syntax: `import packageName.ClassName;`
   - Example: `import java.util.Scanner;`
   - Placed after the package declaration (if present) and before class definition.

3. **Class Definition:**
   - Every Java program must have at least one class.
   - Defines the blueprint for creating objects.
   - Syntax:
     ```java
     public class ClassName {
         // Class members (fields & methods)
     }
     ```
   - More details in **Section 4: Java Class Naming Conventions**.

4. **Fields (Class Variables):**
   - Represent the state or attributes of an object.
   - Declared within the class but outside any method.
   - Syntax: `dataType variableName;`
   - Example: `int age; String name;`
   - More details in **Section 5: Java Variable Naming Conventions**.

5. **Methods:**
   - Define the behavior or actions of an object.
   - Contain executable code that operates on data.
   - Syntax:
     ```java
     accessModifier returnType methodName(parameterList) {
         // Method body (statements)
     }
     ```
   - Example:
     ```java
     public void greet(String name) {
         System.out.println("Hello, " + name + "!");
     }
     ```
   - More details in **Section 6: Java Method Naming Conventions**.

6. **Main Method:**
   - The entry point of execution for every Java application.
   - Must have the exact signature: `public static void main(String[] args)`
   - Example:
     ```java
     public class HelloWorld {
         public static void main(String[] args) {
             System.out.println("Hello, World!");
         }
     }
     ```

**Example:**

```java
package com.example.shapes;

import java.util.Scanner;

public class Circle {
    double radius;

    public double calculateArea() {
        return 3.14 * radius * radius;
    }

    public static void main(String[] args) {
        Scanner input = new Scanner(System.in);
        System.out.print("Enter radius: ");
        double r = input.nextDouble();

        Circle circle = new Circle();
        circle.radius = r;
        double area = circle.calculateArea();

        System.out.println("Area of the circle: " + area);
    }
}
```

---
## Section 2: Importance of Naming Conventions

Using consistent and meaningful names for your code elements significantly improves code readability, maintainability, and collaboration. 

- **Readability:** Clear names make code self-documenting and easier to understand.
- **Maintainability:** Consistent naming makes it easier to modify and debug code.
- **Collaboration:** Standard conventions ensure everyone on the team understands and contributes effectively.

---
## Section 3: Java Package Naming Conventions

Package names are written in all lowercase to avoid potential name collisions with classes or interfaces. Here's the recommended structure:

- **Reverse domain name:** Start with the top-level domain (e.g., `com`, `org`, `net`), followed by your organization's domain, and then project-specific packages.
- **Descriptive and specific:** Clearly indicate the purpose and contents of the package.

**Examples:**

- `com.google.maps`
- `org.apache.commons.io`
- `com.example.ecommerce.products`

---
## Section 4: Java Class Naming Conventions

Class names are crucial for representing objects and concepts in your code. Follow these conventions:

- **PascalCase:** Capitalize the first letter of each word.
- **Nouns or noun phrases:**  Describe the type of object the class represents.
- **Meaningful and specific:** Choose names that clearly convey the purpose of the class.

**Examples:**

- `Customer`
- `ShoppingCart`
- `ProductCatalog`
- `DatabaseConnectionManager`

---
## Section 5: Java Variable Naming Conventions

Choosing descriptive variable names is vital for understanding the purpose of data in your code. Follow these guidelines:

- **camelCase:**  Start with a lowercase letter, capitalize the first letter of subsequent words.
- **Descriptive and meaningful:** Clearly indicate the purpose or content of the variable.
- **Avoid single-letter names (except for loop counters):** Use descriptive names even for temporary variables.

**Examples:**

- `customerName`
- `productPrice`
- `quantityInStock`
- `databaseConnectionUrl`

**Constants:**

- **UPPERCASE_WITH_UNDERSCORES:** Use uppercase letters with underscores for constant values.
- **Declare as `final`:**  Prevent accidental modification.

**Examples:**

- `PI`
- `MAX_USERS`
- `DATABASE_URL`

---
## Section 6: Java Method Naming Conventions

Method names describe the actions performed by your code. Follow these practices:

- **camelCase:** Similar to variable naming.
- **Verbs or verb phrases:** Indicate the action the method performs.
- **Concise and specific:** Clearly communicate the method's purpose.

**Examples:**

- `calculateTotal()`
- `getProductById()`
- `updateCustomerAddress()`
- `openConnection()`

**Getters and Setters (Accessors and Mutators):**

- **Prefix with "get" and "set" for instance variables:**  Follow JavaBean conventions for consistency.

**Examples:**

- `getName()`
- `setName(String name)`
- `getAge()`
- `setAge(int age)`

---
## Resources

- [Oracle Java Code Conventions](https://www.oracle.com/technetwork/java/codeconvtoc-136057.html)
- [Google Java Style Guide](https://google.github.io/styleguide/javaguide.html)

By adhering to these naming conventions and understanding the basic structure of a Java program, you lay a strong foundation for writing clean, maintainable, and industry-standard Java code. 
