## Java Data Types

### Introduction

Data types define the kind of values a variable can hold and the operations that can be performed on them. Understanding data types is fundamental to programming in Java, as it ensures type safety and predictable behavior. 

In Java, we can broadly categorize data types as:

* **Primitive Data Types:** These represent simple values like numbers, characters, and boolean values. They are predefined by the language and stored directly in memory.

* **Reference Data Types:** These represent complex data structures and objects. They store references (memory addresses) to the actual data, which is located elsewhere in memory.

---

## Primitive Data Types

Java provides eight primitive data types:

| Data Type | Size (bits) | Description | Range | Example |
|---|---|---|---|---|
| `byte` | 8 | Represents a signed byte (integer) | -128 to 127 | `byte age = 25;` |
| `short` | 16 | Represents a signed short integer | -32,768 to 32,767 | `short temperature = -5;` |
| `int` | 32 | Represents a signed integer | -2,147,483,648 to 2,147,483,647 | `int population = 800000000;` |
| `long` | 64 | Represents a signed long integer | -9,223,372,036,854,775,808 to 9,223,372,036,854,775,807 | `long nationalDebt = 123456789012345L;` |
| `float` | 32 | Represents a single-precision floating-point number | Approximately ±3.40282347E+38F | `float price = 29.99F;` |
| `double` | 64 | Represents a double-precision floating-point number | Approximately ±1.79769313486231570E+308 | `double pi = 3.141592653589793;` |
| `char` | 16 | Represents a single Unicode character | '\u0000' (0) to '\uffff' (65,535) | `char grade = 'A';` |
| `boolean` | 1 (JVM dependent) | Represents a true or false value | `true` or `false` | `boolean isJavaFun = true;` |

**Note:** JVM may use different sizes for boolean, but for storage efficiency, it mostly uses 1 bit.

**Example Code:**

```java
public class PrimitiveDataTypes {
    public static void main(String[] args) {
        byte age = 25;
        int population = 800000000;
        double price = 29.99;
        char grade = 'A';
        boolean isJavaFun = true;

        System.out.println("Age: " + age);
        System.out.println("Population: " + population);
        System.out.println("Price: " + price);
        System.out.println("Grade: " + grade);
        System.out.println("Is Java Fun? " + isJavaFun);
    }
}
```

**Reference:**

* [Java Primitive Data Types](https://docs.oracle.com/javase/tutorial/java/nutsandbolts/datatypes.html)

---

## Reference Data Types

Reference data types don't store the actual data directly. Instead, they store a reference (memory address) pointing to the location where the data is stored. 

Reference types include:

* **Classes:** User-defined blueprints for creating objects, encapsulating data (fields) and methods.
* **Arrays:** Fixed-size containers that store elements of the same data type.
* **Interfaces:** Contracts defining methods that classes must implement.
* **Enums:** Special classes representing a fixed set of constants.
* **String:** Immutable sequences of characters.

**Example Code:**

```java
public class ReferenceDataTypes {
    public static void main(String[] args) {
        // String
        String message = "Hello, Java!";

        // Array
        int[] numbers = {1, 2, 3, 4, 5};

        // Class instance
        Dog myDog = new Dog("Buddy", "Golden Retriever");

        System.out.println("Message: " + message);
        System.out.println("First number: " + numbers[0]);
        System.out.println("Dog's name: " + myDog.getName());

    }
}

class Dog {
    private String name;
    private String breed;

    public Dog(String name, String breed) {
        this.name = name;
        this.breed = breed;
    }

    public String getName() {
        return name;
    }
}
```

**Reference:**

* [Java Data Types](https://www.w3schools.com/java/java_data_types.asp)

---

## Objects and Class Instances

In object-oriented programming, an **object** is a real-world entity with state and behavior. A **class** is a blueprint or template for creating objects. It defines the data and methods that the objects of that class will have.

* **Class:** Defines the structure and behavior of an object.
* **Object:** An instance of a class, representing a specific entity.

**Example:**

```java
// Class definition
class Car {
    // Fields (data)
    String brand;
    String model;
    int year;

    // Constructor (used to create objects)
    public Car(String brand, String model, int year) {
        this.brand = brand;
        this.model = model;
        this.year = year;
    }

    // Method (behavior)
    public void startEngine() {
        System.out.println("Engine started!");
    }
}

public class Main {
    public static void main(String[] args) {
        // Creating objects (instances of the Car class)
        Car myCar = new Car("Toyota", "Camry", 2022);
        Car anotherCar = new Car("Honda", "Civic", 2021);

        // Accessing object data and methods
        System.out.println(myCar.brand + " " + myCar.model); // Output: Toyota Camry
        myCar.startEngine(); // Output: Engine started!
    }
}
```

In this example:

* `Car` is the class.
* `myCar` and `anotherCar` are objects (instances) of the `Car` class.

**Reference:**

* [Objects and Classes in Java](https://www.tutorialspoint.com/java/java_object_classes.htm)

---

## Null Reference

In Java, a reference variable can hold a special value called `null`. It indicates that the reference variable is not pointing to any valid object.

**Example:**

```java
String name = null;
```

Here, the `name` variable is declared as a `String` reference, but it doesn't refer to any string object yet. It holds the value `null`.

**Important:**

Attempting to access members (fields or methods) of a `null` reference will result in a `NullPointerException`.

**Example:**

```java
String message = null;
int length = message.length(); // Throws NullPointerException
```

---

## Null Pointer Exception Mitigation Methods

**1. Null Checks:**

Always check for `null` before accessing objects:

```java
if (message != null) {
    int length = message.length(); 
}
```

**2. String Methods:**

Use methods like `equals()` and `equalsIgnoreCase()` carefully:

```java
// Instead of:
if (name.equals("John")) { ... }

// Use:
if ("John".equals(name)) { ... }
```

**3. Ternary Operator:**

Concisely handle null checks:

```java
int length = (message != null) ? message.length() : 0;
```

**4. Optional Class (Java 8 and above):**

Represents a container that may or may not contain a non-null value.

```java
Optional<String> optionalName = Optional.ofNullable(name);
int length = optionalName.map(String::length).orElse(0); 
```

---

## Optional Class

Introduced in Java 8, the `Optional` class provides a way to handle null values more elegantly and reduce the risk of `NullPointerExceptions`.

**Creating Optional Objects:**

* `Optional.of(value)`: Creates an `Optional` with a non-null value. Throws `NullPointerException` if `value` is null.
* `Optional.ofNullable(value)`: Creates an `Optional` with the given value, or an empty `Optional` if `value` is null.
* `Optional.empty()`: Creates an empty `Optional`.

**Example:**

```java
String name = "John";
Optional<String> optionalName = Optional.ofNullable(name); // Contains "John"

String nullName = null;
Optional<String> optionalNullName = Optional.ofNullable(nullName); // Empty Optional
```

**Methods for Working with Optional:**

* `isPresent()`: Returns `true` if the `Optional` contains a value, `false` otherwise.
* `get()`: Returns the value if present, otherwise throws `NoSuchElementException`.
* `orElse(other)`: Returns the value if present, otherwise returns `other`.
* `orElseGet(supplier)`: Returns the value if present, otherwise invokes `supplier` and returns the result.
* `orElseThrow(exceptionSupplier)`: Returns the value if present, otherwise throws an exception created by `exceptionSupplier`.
* `map(function)`: Applies the given function to the value if present.
* `flatMap(function)`: Similar to `map`, but the function should return an `Optional`.

**Example:**

```java
public class OptionalExample {
    public static void main(String[] args) {
        String name = "John Doe";
        Optional<String> optName = Optional.ofNullable(name);

        if (optName.isPresent()) {
            System.out.println("Name is: " + optName.get());
        } else {
            System.out.println("Name is not present.");
        }

        String defaultName = optName.orElse("Unknown");
        System.out.println("Default name: " + defaultName);
    }
}
```

**Reference:**

* [Optional Class in Java](https://docs.oracle.com/javase/8/docs/api/java/util/Optional.html)

---
