## Java: Abstract Classes and Interfaces

Welcome to this lecture on two fundamental concepts in Java: abstract classes and interfaces.  We will delve into their definitions, functionalities, differences, and practical applications.


###  1. Abstract Classes

#### 1.1 Introduction

An abstract class in Java is a class that cannot be instantiated and may contain abstract methods (methods without a body). It serves as a blueprint for other classes, defining a common structure and behavior that subclasses must implement. 

**Key characteristics:**

* Declared using the `abstract` keyword.
* Cannot be instantiated directly.
* Can have both abstract and concrete methods.
* Abstract methods must be implemented by subclasses.

#### 1.2 Example

Let's consider an example of shaping a hierarchy for different types of animals:

```java
public abstract class Animal { // Abstract class

    // Abstract method (no implementation)
    public abstract void makeSound(); 

    // Concrete method (with implementation)
    public void sleep() {
        System.out.println("Zzz...");
    }
}

public class Dog extends Animal { 

    @Override
    public void makeSound() { // Implementing the abstract method
        System.out.println("Woof!");
    }
}

public class Cat extends Animal { 

    @Override
    public void makeSound() { // Implementing the abstract method
        System.out.println("Meow!");
    }
}
```

In this example, `Animal` is an abstract class. It defines a common method `sleep()` and an abstract method `makeSound()`. Subclasses `Dog` and `Cat` extend `Animal` and are obligated to provide concrete implementations for the `makeSound()` method.

#### 1.3 When to use abstract classes

* You want to share code among several closely related classes.
* You expect that classes that extend your abstract class will have many common methods or fields, or require access modifiers other than public (such as protected and private).

You can learn more about abstract classes in the official Java documentation: [https://docs.oracle.com/javase/tutorial/java/javaOO/abstract.html](https://docs.oracle.com/javase/tutorial/java/javaOO/abstract.html)


### 2. Interfaces

#### 2.1 Introduction

An interface in Java is a contract that defines a set of methods that a class must implement. It specifies the "what" (the methods) but not the "how" (the implementation). Interfaces provide a way to achieve abstraction and polymorphism in Java.

**Key characteristics:**

* Declared using the `interface` keyword.
* Cannot be instantiated directly.
* Contains only method signatures (no method bodies).
* A class can implement multiple interfaces.

#### 2.2 Example

Let's imagine designing a system where different vehicles can be started and stopped:

```java
public interface Vehicle {

    void start(); // Method signature, no body
    void stop(); // Method signature, no body
}

public class Car implements Vehicle {

    @Override
    public void start() { 
        System.out.println("Starting the car...");
    }

    @Override
    public void stop() { 
        System.out.println("Stopping the car...");
    }
}

public class Motorcycle implements Vehicle {

    @Override
    public void start() { 
        System.out.println("Starting the motorcycle...");
    }

    @Override
    public void stop() { 
        System.out.println("Stopping the motorcycle...");
    }
}
```

Here, `Vehicle` is an interface that defines two methods: `start()` and `stop()`. The classes `Car` and `Motorcycle` implement the `Vehicle` interface and provide their own implementations for the defined methods.

#### 2.3 When to use interfaces:

* You want to specify the behavior of a particular data type, but not concerned about who implements its behavior.
* You need to specify the behavior of a data type, but it is applicable to multiple inheritance hierarchies.
* You want to take advantage of polymorphism.


You can delve deeper into interfaces in the official Java documentation: [https://docs.oracle.com/javase/tutorial/java/concepts/interface.html](https://docs.oracle.com/javase/tutorial/java/concepts/interface.html) 


### 3. Abstract Class vs. Interface: Pros and Cons

#### 3.1 Abstract Class

**Pros:**

* Can have both abstract and concrete methods, allowing for code reusability.
* Can have instance variables to store the state of an object.
* Can use access modifiers to control visibility.

**Cons:**

* A class can only extend one abstract class (single inheritance limitation).
* Modifications to an abstract class can break existing subclasses.

#### 3.2 Interface

**Pros:**

* A class can implement multiple interfaces, enabling multiple inheritance of type.
* Provides a higher level of abstraction, focusing on "what" rather than "how."
* Promotes loose coupling between classes.

**Cons:**

* All methods must be implemented by the implementing class.
* Cannot have instance variables.
* All methods are implicitly public and abstract.

#### 3.3 Choosing the Right One

The choice between an abstract class and an interface depends on the specific requirements of your program. Here’s a table that summarizes the key differences and can aid in decision making:

| Feature          | Abstract Class       | Interface           |
|------------------|-----------------------|---------------------|
| Keyword          | `abstract`           | `interface`          |
| Instantiation    | No                  | No                 |
| Multiple inheritance | No                 | Yes                |
| Method types     | Abstract and concrete | Abstract (Java 8 and later can have default and static methods) |
| Fields           | Yes                  | No (constant fields only) |
| Purpose          | Define a template    | Define a contract  |


In general:

* **Use an abstract class when you want to provide a common implementation for a set of subclasses.**
* **Use an interface when you want to define a contract that multiple unrelated classes can adhere to.**

### 4. Conclusion

Abstract classes and interfaces are powerful tools in Java for achieving abstraction, code reusability, and polymorphism. Understanding their nuances and tradeoffs will empower you to design more robust and flexible software systems.
