## Java Fundamentals: Structure and Naming Conventions

Welcome to this introductory course on Java programming! In this session, we will delve into the fundamental aspects of Java program structure and naming conventions. Understanding these concepts is crucial for writing clean, readable, and maintainable Java code.

---

## 1. Java Program Structure

A typical Java program consists of several key elements organized in a specific structure. This structure ensures that the code is compiled and executed correctly. Let's explore each element:

**1.1. Package Declaration (Optional):**
- Packages are used to group related classes and interfaces together, providing a hierarchical organization for your code.
- The package declaration appears at the very beginning of the Java file. 
- **Syntax:** `package packageName;`
- **Example:** `package com.example.myapp;`

**1.2. Import Statements (Optional):**
- Import statements allow you to use classes from other packages without explicitly using their fully qualified names.
- They appear after the package declaration (if present) and before the class definition.
- **Syntax:** `import packageName.ClassName;`
- **Example:** `import java.util.Scanner;`

**1.3. Class Definition:**
- Every Java program must have at least one class.
- The class definition defines the blueprint for creating objects (instances of the class).
- **Syntax:**
  ```java
  public class ClassName {
      // Class members (fields and methods)
  }
  ```
- **Example:**
  ```java
  public class HelloWorld {
      // Class members
  }
  ```

**1.4. Fields (Variables):**
- Fields represent the data associated with a class or an object of the class.
- They are declared within the class body but outside any method.
- **Syntax:** `dataType variableName;`
- **Example:** `int age;`

**1.5. Methods:**
- Methods define the actions that a class or an object can perform.
- They contain the program logic.
- **Syntax:**
  ```java
  returnType methodName(parameterList) {
      // Method body
  }
  ```
- **Example:**
  ```java
  public static void main(String[] args) {
      // Method body
  }
  ```

**1.6. Comments:**
- Comments are used to explain the code and make it more understandable.
- They are ignored by the compiler.
- **Types:**
  - Single-line comments: `// This is a comment`
  - Multi-line comments: `/* This is a multi-line comment */`

**Example Java Program:**
```java
package com.example.myapp;

import java.util.Scanner;

public class Greeting {
    String name; // Field to store the name

    public static void main(String[] args) {
        Scanner input = new Scanner(System.in);
        System.out.print("Enter your name: ");
        String name = input.nextLine();

        System.out.println("Hello, " + name + "!");
    }
}
```

**Further Reading:**
- [Oracle Java Tutorials - Getting Started](https://docs.oracle.com/java/tutorial/getStarted/index.html)
- [Head First Java, 2nd Edition](https://www.amazon.com/Head-First-Java-Kathy-Sierra/dp/0596009208)

---

## 2. Java Naming Conventions

Naming conventions are essential for writing code that is easy to read, understand, and maintain. While the compiler doesn't enforce strict rules, adhering to these conventions is considered best practice in the Java community.

**2.1. General Naming Conventions:**
- **Case Sensitivity:** Java is case-sensitive, meaning `myVariable` and `myvariable` are different.
- **Meaningful Names:** Choose names that clearly describe the purpose of the identifier (variable, method, class, etc.).
- **No Reserved Keywords:** Avoid using Java reserved keywords (e.g., `public`, `class`, `int`) as identifiers.

**2.2. Specific Naming Conventions:**

| Identifier Type | Convention | Example |
|---|---|---|
| **Packages** | All lowercase, domain name reversed, separated by dots | `com.example.myproject` |
| **Classes & Interfaces** | PascalCase (start each word with uppercase) | `MyClass`, `Runnable` |
| **Methods** | camelCase (start first word lowercase, then capitalize each subsequent word) | `calculateSum()`, `getFirstName()` |
| **Variables** | camelCase (same as methods) | `counter`, `studentName` |
| **Constants** | All uppercase, words separated by underscores | `PI`, `MAX_VALUE` |

**Example:**
```java
package com.example.myapp;

public class Student {
    private String studentName;
    private int studentAge;

    public String getStudentName() {
        return studentName;
    }

    public void setStudentName(String name) {
        studentName = name;
    }

    // ... other methods ...
}
```

---

## 3. Java Package Naming Conventions

Packages are used to organize related classes and interfaces. Proper package naming helps avoid naming conflicts and promotes code reusability.

**3.1. Reverse Domain Name:**
- Use the reverse of your internet domain name to ensure uniqueness.
- For example, if your domain is `example.com`, your package name would start with `com.example`.

**3.2. Lowercase Letters:**
- Use all lowercase letters for package names.
- Separate words with dots (`.`), not underscores (`_`).

**3.3. Examples:**

```
com.example.myproject
org.apache.commons.lang3
net.sourceforge.mylibrary
```

---

## 4. Java Class Naming Conventions

Class names play a significant role in code readability and understanding the purpose of the class.

**4.1. PascalCase:**
- Use PascalCase, where the first letter of each word is capitalized.
- Examples: `MyClass`, `StudentRecord`, `Car`.

**4.2. Nouns or Noun Phrases:**
- Use nouns or noun phrases to describe the class's responsibility.
- The class name should reflect what the class represents.

**4.3. Examples:**
```java
public class User {
    // ...
}

public class ProductCatalog {
    // ...
}
```

---

## 5. Java Variable Naming Conventions

Variable names should clearly indicate the data they hold and follow established conventions.

**5.1. camelCase:**
- Use camelCase, starting with a lowercase letter and capitalizing subsequent words.
- Examples: `counter`, `studentName`, `totalPrice`.

**5.2. Descriptive and Meaningful:**
- Choose names that clearly indicate the variable's purpose and the data it stores.

**5.3. Avoid Single-Letter Names (Exceptions Exist):**
- In most cases, avoid single-letter variable names except for loop counters (e.g., `i`, `j`) or temporary variables within a short scope.

**5.4. Constants:**
- For constants (declared using `final`), use all uppercase letters and separate words with underscores (`_`).

**5.5. Examples:**
```java
int numberOfStudents = 25;
String customerAddress = "123 Main St";
final double PI = 3.14159;
```

---

## 6. Java Method Naming Conventions

Method names should clearly describe the actions they perform.

**6.1. camelCase:**
- Similar to variable names, use camelCase for method names.

**6.2. Verbs or Verb Phrases:**
- Use verbs or verb phrases to describe the method's action.
- Examples: `calculateArea()`, `getStudentName()`, `isValidInput()`.

**6.3. Getters and Setters:**
- For methods that get and set the values of instance variables, use `get` and `set` prefixes, respectively.
- Example: `getName()`, `setName(String name)`.

**6.4. Examples:**
```java
public void calculateSum(int a, int b) {
    // ...
}

public String formatDateString(Date date) {
    // ...
}

public boolean isEvenNumber(int number) {
    // ...
}
```

---
