# Java Course Material

Welcome to the Java programming course. This presentation covers the history, evolution, and features of all major Java versions. We'll explore significant updates, improvements, and code examples relevant to each version. Additionally, references to documentation will be provided for deeper understanding.

---

## Table of Contents

1. [Introduction to Java](#introduction-to-java)
2. [Java Versions Overview](#java-versions-overview)
3. [Java SE 1.0](#java-se-10)
4. [Java SE 1.1](#java-se-11)
5. [Java SE 1.2 (Java 2)](#java-se-12-java-2)
6. [Java SE 5.0](#java-se-50)
7. [Java SE 6](#java-se-6)
8. [Java SE 7](#java-se-7)
9. [Java SE 8](#java-se-8)
10. [Java SE 9](#java-se-9)
11. [Java SE 10](#java-se-10)
12. [Java SE 11](#java-se-11)
13. [Java SE 12](#java-se-12)
14. [Java SE 13](#java-se-13)
15. [Java SE 14](#java-se-14)
16. [Java SE 15](#java-se-15)
17. [Java SE 16](#java-se-16)
18. [Java SE 17](#java-se-17)
19. [References](#references)

---

## Introduction to Java

Java is a versatile and powerful programming language used for developing a wide range of applications from mobile apps to enterprise solutions. It's known for its platform independence, object-oriented features, and strong community support.

### Key Characteristics
- **Write Once, Run Anywhere (WORA):** Java programs can run on any device equipped with the Java Virtual Machine (JVM).
- **Object-Oriented:** Emphasizes encapsulation, inheritance, and polymorphism.
- **Robust and Secure:** Features such as exception handling and garbage collection make Java reliable.
- **Multithreaded:** Facilitates concurrent execution of two or more threads for maximum CPU utilization.

---

## Java Versions Overview

Java has evolved significantly since its initial release, with each version introducing new libraries, tools, and improvements. In this section, we will delve into each major version, exploring new features and enhancements.

---

## Java SE 1.0

**Release Date:** January 1996

- **Initial Release:** The original version with basic features.
- **Core Features:** Basic language structures, AWT for GUI programming.
- **Publications:** [Original Java White Paper](https://archive.org/details/javawhitepaper)

### Code Example
```java
public class HelloWorld {
    public static void main(String[] args) {
        System.out.println("Hello, World!");
    }
}
```

---

## Java SE 1.1

**Release Date:** February 1997

- **Key Features:**
  - JDBC for database connectivity.
  - Inner classes and JavaBeans.
  - RMI (Remote Method Invocation).

### Code Example
```java
// RMI example
import java.rmi.*;

public interface MyRemote extends Remote {
    String sayHello() throws RemoteException;
}
```

---

## Java SE 1.2 (Java 2)

**Release Date:** December 1998

- **Significant Changes:**
  - Swing introduced for GUI components.
  - Collections framework.
  - Improved JVM performance.

### Code Example
```java
import javax.swing.*;

public class MyFrame {
    public static void main(String[] args) {
        JFrame frame = new JFrame("Hello, Java 2!");
        JLabel label = new JLabel("Hello, World!", JLabel.CENTER);
        frame.add(label);
        frame.setSize(300, 300);
        frame.setVisible(true);
    }
}
```

---

## Java SE 5.0

**Release Date:** September 2004

- **Major Features:**
  - Generics introduced.
  - Enhanced for loop.
  - Autoboxing/unboxing.
  - Annotations and Enumerations.
  
### Code Example
```java
// Generics example
import java.util.ArrayList;
import java.util.List;

public class GenericExample {
    public static void main(String[] args) {
        List<Integer> numbers = new ArrayList<>();
        numbers.add(10);
        numbers.add(20);
        
        for (Integer number : numbers) {
            System.out.println(number);
        }
    }
}
```

---

## Java SE 6

**Release Date:** December 2006

- **Enhancements:**
  - Improved Web Service support.
  - Scripting language support via JSR 223.
  - Performance improvements in JVM.
  
### Code Example
```java
// JSR 223 Scripting example
import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;

public class ScriptEngineExample {
    public static void main(String[] args) throws Exception {
        ScriptEngineManager manager = new ScriptEngineManager();
        ScriptEngine engine = manager.getEngineByName("JavaScript");
        engine.eval("print('Hello, Java SE 6!')");
    }
}
```

---

## Java SE 7

**Release Date:** July 2011

- **Key Features:**
  - Switch statements with strings.
  - Automatic resource management (try-with-resources).
  - Improved type inference for generic instance creation.
  
### Code Example
```java
// Try-with-resources example
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

public class TryWithResourcesExample {
    public static void main(String[] args) {
        try (BufferedReader reader = new BufferedReader(new FileReader("file.txt"))) {
            System.out.println(reader.readLine());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
```

---

## Java SE 8

**Release Date:** March 2014

- **Significant Features:**
  - Lambda expressions and functional interfaces.
  - Stream API for bulk data operations on collections.
  - Default and static methods in interfaces.
  - New Date and Time API.
  
### Code Example
```java
// Lambda expression and Stream API example
import java.util.Arrays;
import java.util.List;

public class LambdaStreamExample {
    public static void main(String[] args) {
        List<String> names = Arrays.asList("Alice", "Bob", "Charlie");
        names.stream()
             .filter(name -> name.startsWith("A"))
             .forEach(System.out::println);
    }
}
```

---

## Java SE 9

**Release Date:** September 2017

- **Core Features:**
  - Project Jigsaw for modular system.
  - JShell, the interactive Java shell.
  - Improved Javadoc with search capabilities.
  
### Code Example
```java
// Module example
module com.myapp {
    requires java.base;
}
```

---

## Java SE 10

**Release Date:** March 2018

- **Primary Feature:**
  - Local-variable type inference with `var`.
  
### Code Example
```java
public class VarExample {
    public static void main(String[] args) {
        var message = "Hello, Java 10!";
        System.out.println(message);
    }
}
```

---

## Java SE 11

**Release Date:** September 2018

- **Major Enhancements:**
  - New HTTP Client API.
  - Local-Variable Syntax for Lambda Parameters.
  - Launch single-file source-code programs.
  
### Code Example
```java
// HTTP Client example
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.net.URI;

public class HttpClientExample {
    public static void main(String[] args) throws Exception {
        HttpClient client = HttpClient.newHttpClient();
        HttpRequest request = HttpRequest.newBuilder()
            .uri(new URI("https://example.com"))
            .build();
        
        HttpResponse<String> response = client.send(request, HttpResponse.BodyHandlers.ofString());
        System.out.println(response.body());
    }
}
```

---

## Java SE 12

**Release Date:** March 2019

- **Notable Features:**
  - Switch expressions (preview feature).
  - JVM Constants API.
  
### Code Example
```java
// Switch expression example
public class SwitchExpressionExample {
    public static void main(String[] args) {
        String day = "MONDAY";
        int numLetters = switch (day) {
            case "MONDAY", "FRIDAY", "SUNDAY" -> 6;
            case "TUESDAY" -> 7;
            case "THURSDAY", "SATURDAY" -> 8;
            default -> {
                throw new IllegalStateException("Invalid day: " + day);
            }
        };
        System.out.println(numLetters);
    }
}
```

---

## Java SE 13

**Release Date:** September 2019

- **Features:**
  - Text blocks (preview feature).
  - Dynamic CDS Archives.
  
### Code Example
```java
// Text blocks example
public class TextBlockExample {
    public static void main(String[] args) {
        String json = """
                      {
                          "name": "John",
                          "age": 30
                      }
                      """;
        System.out.println(json);
    }
}
```

---

## Java SE 14

**Release Date:** March 2020

- **Highlights:**
  - Pattern Matching for `instanceof`.
  - Records (preview feature).
  - Switch Expressions become standard.
  
### Code Example
```java
// Record example
public record Point(int x, int y) {}

public class RecordExample {
    public static void main(String[] args) {
        Point point = new Point(3, 4);
        System.out.println(point);
    }
}
```

---

## Java SE 15

**Release Date:** September 2020

- **Key Features:**
  - Records (second preview).
  - Sealed classes (preview feature).
  - Hidden classes for frameworks.
  
### Code Example
```java
// Sealed class example
public abstract sealed class Shape permits Circle, Rectangle {}

final class Circle extends Shape {}
final class Rectangle extends Shape {}

public class SealedClassExample {
    public static void main(String[] args) {
        Shape shape = new Circle();
        System.out.println(shape);
    }
}
```

---

## Java SE 16

**Release Date:** March 2021

- **Enhancements:**
  - Records become final.
  - Pattern Matching for `instanceof` becomes standard.
  - Unix-Domain Socket Channels.
  
### Code Example
```java
// Record as final feature
public record Book(String title, String author) {}

public class BookExample {
    public static void main(String[] args) {
        Book book = new Book("Effective Java", "Joshua Bloch");
        System.out.println(book);
    }
}
```

---

## Java SE 17

**Release Date:** September 2021

- **Significant Updates:**
  - Sealed classes become final.
  - Removal of deprecated API elements.
  - Context-specific deserialization filters.

### Code Example
```java
// Final sealed classes example
public sealed class Vehicle permits Car, Truck {}

public final class Car extends Vehicle {}
public final class Truck extends Vehicle {}

public class SealedExample {
    public static void main(String[] args) {
        Vehicle car = new Car();
        System.out.println(car);
    }
}
```

---

## References

- [Oracle's Java SE Documentation](https://docs.oracle.com/en/java/javase/index.html)
- [Java Language and Virtual Machine Specifications](https://docs.oracle.com/javase/specs/)
- [Java Platform, Standard Edition (Java SE) Version History - Wikipedia](https://en.wikipedia.org/wiki/Java_version_history)
- [The Java Tutorials by Oracle](https://docs.oracle.com/javase/tutorial/)
- [Java Code Geeks Articles](https://www.javacodegeeks.com)

---

This course material is just the starting point for understanding Java. Each version's features allow developers to write more efficient, understandable, and maintainable code. Make sure to refer to the provided documentation for further reading and exploration. Happy coding!