# Java Programming Course Material

## 1. Java Program Structure

### Overview
A Java program's structure consists of several key components. At the most basic level, a Java application must include the main method, which serves as the entry point. Java programs are structured as classes, which encapsulate data (fields) and methods to operate on the data.

### Detailed Structure

1. **Package Declaration**: 
   - Optional, but typically the first line in a source file.
   - Organizes classes into namespaces.

   ```java
   package com.example.myapp;
   ```

2. **Import Statements**:
   - Allow the use of classes from other packages.

   ```java
   import java.util.List;
   import java.util.ArrayList;
   ```

3. **Class Declaration**:
   - Defines a class that can contain fields and methods.

   ```java
   public class MyClass {
       // Fields and methods here
   }
   ```

4. **Main Method**:
   - The entry point of any standalone Java application.

   ```java
   public static void main(String[] args) {
       System.out.println("Hello, World!");
   }
   ```

5. **Fields and Methods**:
   - Fields store the data for the class.
   - Methods are blocks of code that perform tasks.

   ```java
   private String name;
   
   public void setName(String name) {
       this.name = name;
   }
   ```

### Code Example

```java
package com.example.helloworld;

public class HelloWorld {
    public static void main(String[] args) {
        System.out.println("Hello, World!");
    }
}
```

### References
- [The Java™ Tutorials - A Short History of Java](https://docs.oracle.com/javase/tutorial/getStarted/history/index.html)
- [Oracle Java Documentation](https://docs.oracle.com/en/java/)

## 2. Java Naming Conventions

### Overview
Naming conventions in Java are critical for maintainability and readability. They help programmers to understand code without reading it in detail.

### General Rules
- **Classes**: Should be nouns, in CamelCase.
- **Methods**: Should be verbs, in camelCase.
- **Variables**: Should be in camelCase.
- **Constants**: Should be in UPPER_CASE.

### Detailed Conventions

#### Class Naming
- Classes should be written in CamelCase.
- Class names should be nouns representing objects such as `Customer`, `Account`, etc.

```java
public class AccountManager {
    // Class logic
}
```

#### Variable Naming
- Variables should be written in camelCase starting with a lowercase letter.
- Should be meaningful and self-explanatory.

```java
int employeeCount;
String firstName;
```

#### Method Naming
- Methods should be written in camelCase.
- Should typically be a verb or verb phrase.

```java
public void calculateInterest() {
    // Method logic
}
```

#### Constants Naming
- Use UPPER_CASE with underscores to separate words.

```java
public static final int MAX_USERS = 100;
public static final String APPLICATION_NAME = "MyApp";
```

### References
- [Java Code Conventions](https://www.oracle.com/java/technologies/javase/codeconventions-introduction.html)

## 3. Java Package Naming Conventions

### Overview
Packages in Java provide a namespace for classes and interfaces to prevent naming conflicts.

### Conventions
- Use your internet domain name, reversed, as a prefix.
- Followed by project-specific structure.

```java
package com.example.projectname.module;
```

### Example
If your domain is `example.com`, and your application is "Payment System":

```java
package com.example.paymentsystem.transactions;
```

### References
- [The Java™ Tutorials - Package Naming Conventions](https://docs.oracle.com/javase/tutorial/java/package/namingpkgs.html)

## 4. Java Class Naming Conventions

### Overview
Classes encapsulate data and behavior in Java and should be named clearly to reflect their purpose.

### Conventions
- Use nouns or noun phrases.
- Each word in the name should start with an uppercase letter.

```java
public class OrderProcessor {
   // Class logic
}
```

### Example

```java
public class UserAccount {
   // Fields and methods
}
```

### References
- [Java Code Conventions - Class and Interface Declarations](https://www.oracle.com/java/technologies/javase/codeconventions-fileorganization.html)

## 5. Java Variable Naming Conventions

### Overview
Variable names should be descriptive and follow the camelCase naming pattern.

### Conventions
- Start with a lowercase letter.
- Use meaningful and understandable names.

```java
int customerAge;
String addressLine1;
```

### Example
```java
double accountBalance;
boolean isActive;
```

### References
- [Java Variable Naming Convention](https://www.geeksforgeeks.org/java-naming-conventions/)

## 6. Java Method Naming Conventions

### Overview
Methods should be indicative of the action they perform and follow camelCase.

### Conventions
- Start with a verb.
- Use camelCase.

```java
public void saveData() {
   // Method logic
}
```

### Example
```java
public boolean isValid() {
   // Validation logic
}
```

### References
- [Java Method Naming Convention](https://www.geeksforgeeks.org/java-method-naming-conventions/)

---

This course material provides a comprehensive guide to the various aspects of Java programming, focusing on structure and naming conventions. Understanding and internalizing these conventions will lead to better code readability, maintenance, and collaboration in professional Java development.