# Java Programming Course Material: Control Flow and Methods

## Table of Contents
1. Java Control Flow Statements
   - Introduction
   - If-Else Statements
   - Switch Statements
   - For Loops
   - For-Each Loops
   - While Loops
   - Do-While Loops
   - Try-Catch Statements
   - Try-with-Resources Statements

2. Invoking Object and Class Methods
   - Method Invocation
   - Code Examples

3. Accessing Object and Class Fields
   - Field Access
   - Code Examples

## 1. Java Control Flow Statements

### Introduction
Control flow statements in Java allow developers to dictate the execution paths based on conditions and loops. They are critical in programming as they determine how and when parts of your code are executed.

For documentation, refer to the [Java Language Specification](https://docs.oracle.com/javase/specs/).

### If-Else Statements
If-Else statements allow you to execute code blocks based on boolean conditions.

```java
int number = 10;
if (number > 0) {
    System.out.println("The number is positive");
} else {
    System.out.println("The number is non-positive");
}
```

- The above example checks if `number` is positive and prints a statement accordingly.
- Reference: Official [Java Tutorials](https://docs.oracle.com/javase/tutorial/java/nutsandbolts/if.html).

### Switch Statements
Switch statements execute code blocks based on the value of a variable.

```java
int month = 8;
String monthName;
switch (month) {
    case 1:  monthName = "January"; break;
    case 2:  monthName = "February"; break;
    // remaining months
    default: monthName = "Invalid month"; break;
}
```

- Switch is typically used for variable matching.
- Reference: [Java Switch Docs](https://docs.oracle.com/javase/tutorial/java/nutsandbolts/switch.html).

### For Loops
For loops allow repeated execution of a code block.

```java
for (int i = 0; i < 5; i++) {
    System.out.println("Iteration: " + i);
}
```

- This loop prints numbers from 0 to 4.
- Reference: [Java For Loop Docs](https://docs.oracle.com/javase/tutorial/java/nutsandbolts/for.html).

### For-Each Loops
Ideal for iterating through collections or arrays.

```java
int[] numbers = {1, 2, 3, 4, 5};
for (int num : numbers) {
    System.out.println(num);
}
```

- Iterates directly over an array/collection.
- Reference: [Enhanced For Loop](https://docs.oracle.com/javase/8/docs/technotes/guides/language/foreach.html).

### While Loops
Executes as long as a condition is true.

```java
int count = 0;
while (count < 5) {
    System.out.println("Count is: " + count);
    count++;
}
```

- Useful for loop execution based on dynamic conditions.
- Reference: [While Loop Guide](https://docs.oracle.com/javase/tutorial/java/nutsandbolts/while.html).

### Do-While Loops
Executes at least once and then based on the condition.

```java
int count = 0;
do {
    System.out.println("Count is: " + count);
    count++;
} while (count < 5);
```

- Always executes at least once.
- Reference: [Do-While Loop Example](https://docs.oracle.com/javase/tutorial/java/nutsandbolts/while.html).

### Switch Statements with Pattern Matching
Pattern matching enhances switch for more complex scenarios. For example, Java 17 introduced pattern matching for switch.

Example:
```java
Object obj = 123;
switch (obj) {
    case Integer i     -> System.out.println("Integer: " + i);
    case String s      -> System.out.println("String: " + s);
    default            -> System.out.println("Unknown type");
}
```

- Simplifies type checking.
- Reference: [JEP 406: Pattern Matching for switch](https://openjdk.java.net/jeps/406).

### Try-Catch Statements
Handle exceptions to prevent program crashes.

```java
try {
    int result = 10 / 0;
} catch (ArithmeticException e) {
    System.out.println("Cannot divide by zero");
}
```

- Catch specific exceptions to manage errors.
- Reference: [Java Exceptions](https://docs.oracle.com/javase/tutorial/essential/exceptions/).

### Try-with-Resources Statements
Automatically closes resources (like files) when done.

```java
try (BufferedReader br = new BufferedReader(new FileReader("file.txt"))) {
    System.out.println(br.readLine());
} catch (IOException e) {
    e.printStackTrace();
}
```

- Closes the resource, avoiding potential memory leaks.
- Reference: [Try-with-Resources Guide](https://docs.oracle.com/javase/tutorial/essential/exceptions/tryResourceClose.html).

## 2. Invoking Object and Class Methods

### Method Invocation
Methods perform specific tasks. Object methods require an instance, while class (static) methods do not.

```java
class MathUtil {
    public int add(int a, int b) {
        return a + b;
    }

    public static int subtract(int a, int b) {
        return a - b;
    }
}

public class Main {
    public static void main(String[] args) {
        MathUtil util = new MathUtil();
        System.out.println("Addition: " + util.add(5, 3)); // Object method

        System.out.println("Subtraction: " + MathUtil.subtract(5, 3)); // Class method
    }
}
```

- Methods can be invoked using a dot `.` operator.
- Reference: [Java Methods Documentation](https://docs.oracle.com/javase/tutorial/java/javaOO/methods.html).

## 3. Accessing Object and Class Fields

### Field Access
Fields (or attributes) represent data. Object fields are accessed via instances, while class fields are static.

```java
class Car {
    public String model;
    public static int wheels = 4;
}

public class Main {
    public static void main(String[] args) {
        Car myCar = new Car();
        myCar.model = "Tesla";

        System.out.println("Model: " + myCar.model); // Object field
        System.out.println("Wheels: " + Car.wheels); // Class field
    }
}
```

- Fields hold the state of a class/object.
- Reference: [Field Declaration](https://docs.oracle.com/javase/tutorial/java/javaOO/variables.html).

## References
- [Oracle Java Documentation](https://docs.oracle.com/en/java/)
- [Java Programming by Herbert Schildt](https://www.amazon.com/Java-Complete-Reference-Eleventh/dp/1260440230)

This presentation offers a comprehensive look at key Java concepts. To expand your understanding, consider reviewing books and online resources comprehensively.