# Java Course Material: Understanding the Object Class

## Introduction

In Java, the `Object` class is the root class of the class hierarchy. Every class has `Object` as a superclass, either directly or indirectly. This means every class in Java inherits the methods of the `Object` class. Understanding this class is crucial for mastering Java and object-oriented programming.

## The Object Class

The Object class is part of the `java.lang` package and forms the foundation of Java's class concept. It provides several basic methods that all Java objects inherit and can modify.

```markdown
class Object {
    // default constructor
    public Object() { }
    
    // fundamental methods
    public boolean equals(Object obj) { ... }
    public int hashCode() { ... }
    protected Object clone() throws CloneNotSupportedException { ... }
    public String toString() { ... }
    protected void finalize() throws Throwable { ... }
    public final Class<?> getClass() { ... }
    public final void notify() { ... }
    public final void notifyAll() { ... }
    public final void wait(long timeout) throws InterruptedException { ... }
    public final void wait(long timeout, int nanos) throws InterruptedException { ... }
    public final void wait() throws InterruptedException { ... }
}
```

### References:
- [Java Platform API Specification](https://docs.oracle.com/en/java/javase/17/docs/api/java.base/java/lang/Object.html)

## Object Class Methods

### 1. **equals(Object obj)**

The `equals` method determines whether two objects are equal. By default, its implementation in the `Object` class compares the memory addresses of the objects, not the actual content.

- **Usage:** Override `equals` to compare object data.
  
```java
public class Person {
    private String name;
    private int age;

    public Person(String name, int age) {
        this.name = name;
        this.age = age;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) return true;
        if (obj == null || getClass() != obj.getClass()) return false;
        Person person = (Person) obj;
        return age == person.age && Objects.equals(name, person.name);
    }
}
```

### References:
- [Effective Java by Joshua Bloch](https://www.oreilly.com/library/view/effective-java/9780134686097/)
- [Java SE Documentation](https://docs.oracle.com/javase/tutorial/java/IandI/objectclass.html)

### 2. **hashCode()**

The `hashCode` method returns an integer hash code value for an object, used in hashing-based collections like `HashMap`. It must be consistent with the `equals` method, meaning if two objects are equal, they must have the same hash code.

```java
@Override
public int hashCode() {
    return Objects.hash(name, age);
}
```

### References:
- [Java SE Documentation](https://docs.oracle.com/javase/7/docs/api/java/lang/Object.html#hashCode())

### 3. **toString()**

The `toString` method returns a string representation of the object. By default, it returns a string consisting of the class name, the at-sign, and the object's hashcode.

```java
@Override
public String toString() {
    return "Person{name='" + name + "', age=" + age + '}';
}
```

### References:
- [The toString Method](https://docs.oracle.com/javase/tutorial/java/IandI/toString.html)

### 4. **getClass()**

This method returns the runtime class of the object, which can be used for reflection.

```java
Person person = new Person("John", 30);
System.out.println(person.getClass().getName()); // Output: Person
```

### 5. **clone()**

Protected and intended for subclass implementation, it is used to create and return a copy of the object.

### 6. **finalize()**

Invoked by the garbage collector on an object when garbage collection determines that there are no more references to the object.

### 7. **notify(), notifyAll(), wait()**

Methods used for thread communication in a multi-threaded environment, they must be used within synchronized code blocks.

## Object Class Fields

The `Object` class does not declare any explicit fields. However, it is important to understand object identity and memory management, which are implicit aspects covered by the methods provided by the `Object` class.

## Deep Dive: equals and hashCode

**equals** and **hashCode** are critical, especially in collections.

- **Contract:**
  - If two objects are equal according to the `equals()`, then calling `hashCode()` must produce the same integer result for both objects.
  - Override both methods if one is overridden.

```java
public class Car {
    private String model;
    private int year;

    // Override equals
    @Override
    public boolean equals(Object obj) {
        if (this == obj) return true;
        if (!(obj instanceof Car)) return false;
        Car car = (Car) obj;
        return year == car.year && Objects.equals(model, car.model);
    }

    // Override hashCode
    @Override
    public int hashCode() {
        return Objects.hash(model, year);
    }
}
```

### References:
- [Java HashMap](https://docs.oracle.com/javase/tutorial/collections/interfaces/map.html)
- [Object Contract - Overriding equals() and hashCode()](https://docs.oracle.com/javase/8/docs/api/java/lang/Object.html)

## **Conclusion**

The `Object` class is a pivotal part of Java's core if you seek to delve deeper into Java development and grasp the key concepts of OOP. Understanding its methods, especially `equals`, `hashCode`, and `toString`, helps in creating robust Java applications.

---

This presentation covers the essential functionalities provided by the Java `Object` class. For further reading, consult the Java API documentation and recommended books like *Effective Java* by Joshua Bloch.