# Java Course Material: Java Data Types

## Java Data Types Overview

Java data types can be broadly classified into two categories:
- **Primitive Data Types**
- **Reference Data Types**

Understanding these types is crucial as they define the kind of data a variable can store and the operations that can be performed on them.

## Primitive Data Types

Primitive types are the most basic data types available in Java. There are eight primitive data types, each serving a unique purpose:

1. **byte**: 
   - **Size:** 8-bit 
   - **Range:** -128 to 127 
   - **Usage:** Useful in large arrays where memory savings are needed.

   ```java
   byte b = 100;
   ```

2. **short**: 
   - **Size:** 16-bit 
   - **Range:** -32,768 to 32,767 
   - **Usage:** Suitable for saving memory in large arrays.

   ```java
   short s = 10000;
   ```

3. **int**: 
   - **Size:** 32-bit 
   - **Range:** -2^31 to 2^31-1 
   - **Usage:** Default integer type, used broadly throughout Java programs.

   ```java
   int i = 100000;
   ```

4. **long**: 
   - **Size:** 64-bit 
   - **Range:** -2^63 to 2^63-1
   - **Usage:** Used when a wider range than int is needed. Append 'L' to the value.

   ```java
   long l = 100000L;
   ```

5. **float**: 
   - **Size:** 32-bit floating-point 
   - **Usage:** Used for saving memory in large arrays of floating-point numbers; precision is about 6-7 decimal digits.

   ```java
   float f = 234.5f;
   ```

6. **double**: 
   - **Size:** 64-bit floating-point 
   - **Usage:** Default data type for decimal values, generally used in libraries and for precise calculations; precision about 15 decimal digits.

   ```java
   double d = 123.4;
   ```

7. **boolean**: 
   - **Size:** Represents one bit of information (true or false). 
   - **Usage:** Used for simple flags that track true/false conditions.

   ```java
   boolean flag = true;
   ```

8. **char**: 
   - **Size:** 16-bit Unicode character 
   - **Usage:** Used for characters.

   ```java
   char letter = 'A';
   ```

**References**:
- [Java Primitive Data Types - Oracle Documentation](https://docs.oracle.com/javase/tutorial/java/nutsandbolts/datatypes.html)
- "Effective Java" by Joshua Bloch

## Reference Types

Reference types in Java are any objects that reference a memory location of the object. In Java, all objects are dynamically allocated on the heap.

### Examples:

- **Classes**
- **Interfaces**
- **Arrays**

#### Example of a Reference Type (Class)

```java
public class Car {
    String color;
    String model;
    
    Car(String c, String m) {
        this.color = c;
        this.model = m;
    }
}

Car carObject = new Car("Red", "Toyota");
```

#### Arrays as Reference Types

```java
int[] numbers = {1, 2, 3, 4, 5};
String[] names = new String[3];
```

**References**:
- [Java Reference Data Types](https://docs.oracle.com/javase/specs/jls/se8/html/jls-4.html)
- "Java: The Complete Reference" by Herbert Schildt

## Object and Class Instances

### Class

A class is a blueprint from which individual objects are created. It defines the data and the behavior of the represented objects.

### Object

An object is an instance of a class. Objects hold their own distinct state, but they share behavior (method) with the class.

#### Example

```java
public class Student {
    String name;
    int age;

    void display() {
        System.out.println("Name: " + name + ", Age: " + age);
    }
    
}

public class Main {
    public static void main(String[] args) {
        Student student1 = new Student();
        student1.name = "John";
        student1.age = 25;
        student1.display();
    }
}
```

**References**:
- [Java Classes and Objects - Oracle Documentation](https://docs.oracle.com/javase/tutorial/java/javaOO/classes.html)
- "Clean Code" by Robert C. Martin

## Null Reference

A null reference in Java is a reference variable that doesn't point to any object or memory location. Using a null reference as though it points to an object can lead to `NullPointerException`.

#### Example

```java
String str = null;
System.out.println(str.length()); // This will throw NullPointerException
```

**References**:
- [NullPointerException - Oracle Documentation](https://docs.oracle.com/javase/8/docs/api/java/lang/NullPointerException.html)

## Null Pointer Exception Mitigation Methods

To mitigate `NullPointerException`, follow these strategies:

1. **Checking for null explicitly**

   ```java
   if (str != null) {
       System.out.println(str.length());
   }
   ```

2. **Using Ternary Operator**

   ```java
   int length = (str != null) ? str.length() : 0;
   ```

3. **Using Java 8 Optional**

   ```java
   Optional<String> optionalStr = Optional.ofNullable(str);
   optionalStr.ifPresent(s -> System.out.println(s.length()));
   ```

4. **Initializing variables properly**

5. **Avoiding returning null from methods**

**References**:
- "Java Puzzlers: Traps, Pitfalls, and Corner Cases" by Joshua Bloch

## Optional Class

The `Optional` class was introduced in Java 8 to help with null pointer exceptions. It is a container object which may or may not contain a non-null value.

### Examples of Optional Usage

Creating an `Optional`:

```java
Optional<String> optional = Optional.of("Hello, World!");
```

Checking and using `Optional`:

```java
optional.ifPresent(System.out::println);

String defaultValue = optional.orElse("Default Value");
System.out.println(defaultValue);
```

Using `Optional` in method return type:

```java
public Optional<Car> findCarById(Long carId) {
    return Optional.ofNullable(carRepository.get(carId));
}
```

**References**:
- [Optional Class in Java 8 - Oracle Documentation](https://docs.oracle.com/javase/8/docs/api/java/util/Optional.html)
- "Java 8 in Action" by Raoul-Gabriel Urma, Mario Fusco, Alan Mycroft

This presentation should serve as a comprehensive guide to understanding Java data types, highlighting the distinctions and use cases for primitive and reference types, and best practices for dealing with null values.