# Java Comments: An In-Depth Guide

Comments are an essential part of any programming language, providing a way to document the code for better understanding and maintenance. Java, a statically-typed, object-oriented programming language, offers three types of comments: single-line, multi-line, and Javadoc comments. In this course material, we will explore each type in detail, complete with examples and references.

---

## Table of Contents
1. [Introduction to Java Comments](#introduction-to-java-comments)
2. [Single-Line Comments](#single-line-comments)
    - [Syntax](#syntax)
    - [Examples](#examples)
    - [References](#references)
3. [Multi-Line Comments](#multi-line-comments)
    - [Syntax](#syntax-1)
    - [Examples](#examples-1)
    - [References](#references-1)
4. [Javadoc Comments](#javadoc-comments)
    - [Syntax](#syntax-2)
    - [Examples](#examples-2)
    - [References](#references-2)

---

## Introduction to Java Comments

In Java, comments are non-executable statements that serve as annotations for understanding the code. They are ignored by the Java compiler during execution. Proper commenting aids in following the logic of the code, explaining functions or complex algorithms, and providing documentation for other developers or future reference.

---

## Single-Line Comments

### Syntax

Single-line comments in Java start with two forward slashes (`//`). Anything following these slashes on the same line is considered as a comment and is ignored during the compilation process.

### Examples

Single-line comments are typically used for short annotations or to comment out code temporarily.

// This is a single-line comment
```java
public class Main {
    public static void main(String[] args) {
        // Print Hello World to the console
        System.out.println("Hello, World!");

        int sum = 5 + 5; // Adding two numbers
        System.out.println("Sum is: " + sum);

        // System.out.println("This code is commented out");
    }
}
```

In this example:
- Let's developers understand what the `println` function is doing.
- Provides justification for changes or intentions in specific lines of code.

### References

- *Effective Java* by Joshua Bloch - Discusses best practices in writing comments.
- [Oracle Java Documentation](https://docs.oracle.com/javase/tutorial/java/nutsandbolts/comments.html) - Official documentation on Java comments.

---

## Multi-Line Comments

### Syntax

Multi-line comments begin with `/*` and end with `*/`. Everything between these markers is treated as a comment, allowing for longer, paragraph-style documentation.

### Examples

They are useful for commenting out a block of code or providing detailed explanations.

/* This is a multi-line comment */

```java
public class Calculator {
    public static void main(String[] args) {
        /*
          The following code block
          performs basic arithmetic
          operations.
        */
        int x = 10;
        int y = 5;

        // Addition
        int sum = x + y;
        System.out.println("Sum: " + sum);

        // Subtraction
        int difference = x - y;
        System.out.println("Difference: " + difference);
    }
}
```

In this example, the multi-line comment explains the purpose of the subsequent code block.

### References

- *Java: The Complete Reference* by Herbert Schildt - Covers the basics of writing comments.
- [Oracle Java Documentation](https://docs.oracle.com/javase/tutorial/java/nutsandbolts/comments.html) - Official documentation on Java comments.

---

## Javadoc Comments

### Syntax

Javadoc comments start with `/**` and end with `*/`. They are used to create HTML-formatted documentation from the code. Javadoc comments can include tags like `@param` and `@return` to describe method parameters and return values.

### Examples

Javadoc comments are typically placed above classes, methods, and fields.

/** This is a Javadoc comment */

```java
/**
 * The MathUtility class provides static methods
 * for basic math operations.
 */
public class MathUtility {
    /**
     * Adds two integer numbers.
     *
     * @param a the first number
     * @param b the second number
     * @return the sum of a and b
     */
    public static int add(int a, int b) {
        return a + b;
    }
}
```

In this example, the Javadoc comments describe the class and its method in detail, which aids in generating external documentation.

### References

- *Head First Java* by Kathy Sierra & Bert Bates - A beginner-friendly book explaining Javadoc and its usage.
- [Oracle's Javadoc Guide](https://www.oracle.com/technical-resources/articles/java/javadoc-tool.html) - Comprehensive resource for writing Javadoc comments.

---

By understanding and implementing these types of comments, Java developers can write more readable, maintainable, and well-documented code. Properly documented code is paramount in collaborative environments and long-term projects where various stakeholders are involved.