# Java Object-Oriented Programming: Abstract Classes and Interfaces

## Abstract Class

### What is an Abstract Class?
An abstract class in Java is a class that cannot be instantiated on its own and is meant to be subclassed. It can contain both abstract methods (methods without a body) and concrete methods (methods with a body).

#### Key Characteristics:
- **Keyword**: Declared using the `abstract` keyword.
- **Instantiation**: Cannot be instantiated directly.
- **Methods**: Can have both abstract and non-abstract methods.
- **Constructors**: Can have constructors.
- **Access Modifiers**: Can have access modifiers.

#### Example:
```java
abstract class Animal {
    // Abstract method (does not have a body)
    abstract void makeSound();

    // Regular method
    void eat() {
        System.out.println("This animal eats");
    }
}

class Dog extends Animal {
    void makeSound() {
        System.out.println("Woof");
    }
}

public class TestAbstract {
    public static void main(String[] args) {
        Dog dog = new Dog();
        dog.makeSound(); // Outputs: Woof
        dog.eat();       // Outputs: This animal eats
    }
}
```

### References:
- "Effective Java" by Joshua Bloch
- Oracle Java Documentation [Java Abstract Classes](https://docs.oracle.com/javase/tutorial/java/IandI/abstract.html)

## Interface

### What is an Interface?
An interface in Java is a reference type that is similar to a class and is a collection of abstract methods that a class can implement.

#### Key Characteristics:
- **Keyword**: Declared using the `interface` keyword.
- **Instantiation**: Cannot be instantiated.
- **Methods**: All methods are abstract and public by default until Java 8.
- **Fields**: All fields are `public`, `static`, and `final` by default.
- **Multiple Inheritance**: A class can implement multiple interfaces.

#### Example:
```java
interface Animal {
    void makeSound();
}

class Cat implements Animal {
    public void makeSound() {
        System.out.println("Meow");
    }
}

public class TestInterface {
    public static void main(String[] args) {
        Cat cat = new Cat();
        cat.makeSound(); // Outputs: Meow
    }
}
```

### References:
- "Java: The Complete Reference" by Herbert Schildt
- Oracle Java Documentation [Java Interfaces](https://docs.oracle.com/javase/tutorial/java/IandI/createinterface.html)

## Abstract Class vs Interface

### Pros and Cons

#### Abstract Class
**Pros:**
- Can have concrete methods, allowing code sharing among subclasses.
- Can have instance variables.
- Suitable for closely related classes with shared methods or fields.

**Cons:**
- A class can only extend one abstract class due to single inheritance.

#### Interface
**Pros:**
- Supports multiple inheritance as a class can implement multiple interfaces.
- Ideal for defining capabilities that any class can implement regardless of where it resides in the class hierarchy.

**Cons:**
- Can’t provide code implementation until Java 8 (with default methods).
- Can't have instance variables.

### Example of Both:

```java
abstract class Vehicle {
    abstract void start();
    void stop() {
        System.out.println("Vehicle stopped");
    }
}

interface Electric {
    void charge();
}

class Tesla extends Vehicle implements Electric {
    void start() {
        System.out.println("Tesla starting");
    }

    public void charge() {
        System.out.println("Tesla charging");
    }
}

public class Main {
    public static void main(String[] args) {
        Tesla myTesla = new Tesla();
        myTesla.start();  // Outputs: Tesla starting
        myTesla.charge(); // Outputs: Tesla charging
        myTesla.stop();   // Outputs: Vehicle stopped
    }
}
```

### References:
- "Design Patterns: Elements of Reusable Object-Oriented Software" by Erich Gamma et al.
- The Java Programming Language by Ken Arnold, James Gosling, David Holmes

This presentation provides an overview of abstract classes and interfaces in Java, their respective features, benefits, limitations, and suitable use cases along with practical examples to help reinforce the concepts.