## Java Through the Years: A Journey of Innovation

**Course Overview:** This lecture delves into the evolution of Java, exploring major versions and their defining features. We'll navigate through key advancements, illustrated by code examples and supported by external resources for further exploration.

---

### 1. Genesis: Java 1.0 (1996)

**Key Features:**

* **Object-Oriented Programming:** Embraced OOP principles like encapsulation, inheritance, and polymorphism.
* **Platform Independence:** Introduced "Write Once, Run Anywhere" (WORA) capability using the Java Virtual Machine (JVM).
* **Built-in Multithreading:** Enabled concurrent execution of program parts.
* **Security:** Provided a secure execution environment with features like the Security Manager.

**Example: "Hello, World!" in Java 1.0**

```java
public class HelloWorld {
    public static void main(String[] args) {
        System.out.println("Hello, World!");
    }
}
```

**Resources:**

* [The Java Language Environment](https://www.oracle.com/java/technologies/java-history.html)

---

### 2.  Swinging into Action: Java 1.2 (1998)

**Key Features:**

* **Swing GUI Toolkit:** Introduced a more sophisticated and flexible GUI toolkit than AWT.
* **Collections Framework:** Provided standardized data structures for efficient data management.
* **Reflection API:** Allowed inspection and modification of program structure at runtime.

**Example: Swing Button**

```java
import javax.swing.*;

public class SwingExample {
    public static void main(String[] args) {
        JFrame frame = new JFrame("Swing Example");
        JButton button = new JButton("Click Me");
        frame.getContentPane().add(button);
        frame.setSize(300, 200);
        frame.setVisible(true);
    }
}
```

**Resources:**

* [The Java Tutorial: Collections](https://docs.oracle.com/javase/tutorial/collections/)

---

### 3. A Generics Revolution: Java 5 (2004)

**Key Features:**

* **Generics:** Introduced type safety for collections and eliminated many casting requirements.
* **Enhanced for loop:** Provided a simpler syntax for iterating over collections and arrays.
* **Autoboxing/Unboxing:** Enabled automatic conversion between primitive types and their wrapper classes.

**Example: Generics in ArrayList**

```java
import java.util.ArrayList;

public class GenericsExample {
    public static void main(String[] args) {
        ArrayList<String> names = new ArrayList<String>(); // Type-safe ArrayList
        names.add("Alice");
        names.add("Bob");

        for (String name : names) {
            System.out.println(name);
        }
    }
}
```

**Resources:**

* [Java Generics FAQs](https://docs.oracle.com/javase/tutorial/java/generics/index.html)

---

### 4.  Embracing Change: Java SE 7 (2011)

**Key Features:**

* **Diamond Operator (<>):** Reduced verbosity when using generics.
* **Strings in switch:** Allowed the use of strings in switch statements.
* **Try-with-resources:** Simplified exception handling by automatically closing resources.

**Example: Try-with-resources**

```java
public class TryWithResources {
    public static void main(String[] args) {
        try (BufferedReader br = new BufferedReader(new FileReader("file.txt"))) {
            String line;
            while ((line = br.readLine()) != null) {
                System.out.println(line);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
```

**Resources:**

* [JDK 7 Release Notes](https://www.oracle.com/java/technologies/javase/7-relnotes.html)

---

### 5.  Lambda Power: Java SE 8 (2014)

**Key Features:**

* **Lambda Expressions:** Introduced functional programming concepts, making code more concise and expressive.
* **Stream API:**  Enabled powerful data manipulation operations on collections.
* **Date and Time API:** Provided a more comprehensive and modern API for working with dates and times.

**Example: Lambda Expression**

```java
import java.util.Arrays;
import java.util.List;

public class LambdaExample {
    public static void main(String[] args) {
        List<String> names = Arrays.asList("Alice", "Bob", "Charlie");

        // Using a lambda expression to print each name
        names.forEach(name -> System.out.println(name));
    }
}
```

**Resources:**

* [Lambda Expressions](https://docs.oracle.com/javase/tutorial/java/javaOO/lambdaexpressions.html)

---

### 6.  Modules and More: Java SE 9 (2017)

**Key Features:**

* **Modular System (Project Jigsaw):** Introduced a modular structure to the JDK, improving performance and maintainability.
* **JShell (REPL):** Provided an interactive Read-Eval-Print Loop for experimenting with code.
* **Private methods in interfaces:** Allowed for code reuse within interfaces.

**Example: JShell**

```bash
jshell> System.out.println("Hello from JShell!");
Hello from JShell!
```

**Resources:**

* [The Java Platform Module System](https://openjdk.java.net/projects/jigsaw/)

---

### 7.  Evolving Steadily: Java SE 11 (2018) - LTS

**Key Features:**

* **Local-Variable Type Inference (var keyword):** Simplified code by allowing the compiler to infer variable types.
* **New String Methods:** Added useful methods to the String class, like `isBlank()` and `lines()`.
* **HTTP/2 Client API:** Introduced a new HTTP client with support for HTTP/2.

**Example: Local-Variable Type Inference**

```java
public class VarExample {
    public static void main(String[] args) {
        var name = "John Doe"; // Type inferred as String
        System.out.println(name);
    }
}
```

**Resources:**

* [JDK 11 Release Notes](https://www.oracle.com/java/technologies/javase/11-relnotes.html)

---

### 8.  Records and Beyond: Java SE 14 (2020) - LTS

**Key Features:**

* **Records:** Introduced a concise way to define immutable data classes.
* **Switch Expressions:** Enhanced switch statements to be more concise and expressive.
* **Pattern Matching for instanceof:** Improved pattern matching capabilities.

**Example: Records**

```java
public record Person(String firstName, String lastName, int age) {
    // Compact syntax for immutable data class
}
```

**Resources:**

* [JDK 14 Release Notes](https://www.oracle.com/java/technologies/javase/14-relnotes.html)

---

### 9. Looking Ahead:  Java SE 17 (2021) - LTS & Beyond

**Key Features:**

* **Sealed Classes:** Enhanced control over inheritance by restricting which classes can extend them.
* **Pattern Matching for switch (Preview):** Further improved pattern matching in switch expressions.
* **New macOS Rendering Pipeline:** Introduced a new rendering pipeline for macOS using Apple Metal API. 

**Resources:**

* [JDK 17 Release Notes](https://www.oracle.com/java/technologies/javase/17-relnotes.html)
* [OpenJDK](https://openjdk.java.net/)

---

This lecture provided a glimpse into the evolution of Java.  Continuous exploration of new features and updates is essential to harnessing the full power of this versatile language.
