## Java Object-Oriented Programming Design Principles

This lecture will delve into the core principles of Object-Oriented Programming (OOP) within the Java language. We will explore these principles in detail, illustrating them with code examples and references for further exploration.

### What is Object-Oriented Programming (OOP)?

OOP is a programming paradigm that structures software design around **objects**, which are instances of **classes**.  Think of classes as blueprints and objects as the actual structures built from those blueprints. 

**Key Concepts:**

* **Abstraction:** Hiding complex implementation details and showing only essential information to the user. 
* **Encapsulation:** Bundling data (attributes) and methods (functions) that operate on that data into a single unit (class), and controlling access to that data.
* **Inheritance:** Creating new classes (child classes) from existing ones (parent classes), inheriting their properties and behaviors.
* **Polymorphism:**  The ability of an object to take on multiple forms; using a single interface to represent different underlying forms.

### 1. Encapsulation

**Definition:** Encapsulation restricts access to an object's internal state (data) by hiding its implementation details and providing controlled access through methods.

**Benefits:**

* **Data Protection:** Prevents accidental or unauthorized modifications.
* **Code Flexibility:** Internal implementation changes won't affect other parts of the code.
* **Increased Maintainability:** Easier to understand, debug, and modify code.

**Example:**

```java
class BankAccount {
    private double balance; // Private data member

    public double getBalance() { // Public getter method
        return balance;
    }

    public void deposit(double amount) { // Public setter method
        balance += amount;
    }
}
```

**Explanation:** The `balance` variable is private, meaning it can only be accessed within the `BankAccount` class. Access to this data is controlled through public `getBalance` and `deposit` methods.

### 2. Inheritance

**Definition:**  Inheritance enables you to create new classes (child classes or subclasses) based on existing classes (parent classes or superclasses). Child classes inherit properties and methods from their parent classes, promoting code reusability and reducing redundancy.

**Benefits:**

* **Code Reusability:** Avoid rewriting the same code for similar functionalities.
* **Extensibility:** Easily add new features without modifying existing code.
* **Code Organization:** Structures code into hierarchies, improving readability.

**Example:**

```java
class Animal { // Parent class
    public void eat() {
        System.out.println("Animal is eating");
    }
}

class Dog extends Animal { // Child class inheriting from Animal
    public void bark() {
        System.out.println("Dog is barking");
    }
}
```

**Explanation:** The `Dog` class inherits the `eat()` method from the `Animal` class and adds its own `bark()` method.

### 3. Polymorphism

**Definition:** Polymorphism literally means "many forms." In OOP, it refers to the ability of an object to take on different forms.  This is achieved through method overriding and method overloading.

**Types of Polymorphism:**

* **Compile-Time Polymorphism (Method Overloading):** Defining multiple methods with the same name but different parameters in the same class.
* **Run-Time Polymorphism (Method Overriding):** A subclass provides a specific implementation of a method that is already provided by its superclass.

**Benefits:**

* **Code Flexibility:** Write more generic and reusable code.
* **Dynamic Binding:** Method calls are resolved at runtime, allowing for more dynamic behavior.

**Example (Method Overriding):**

```java
class Shape { 
    public void draw() { 
        System.out.println("Drawing a shape"); 
    } 
}

class Circle extends Shape { 
    @Override
    public void draw() { 
        System.out.println("Drawing a circle"); 
    } 
}
```

**Explanation:** Both `Shape` and `Circle` have a `draw()` method. When a `Circle` object calls `draw()`, the overridden method in the `Circle` class is executed.

### 4. Abstraction

**Definition:** Abstraction focuses on hiding the complex implementation details of an object and presenting only the essential information to the user.

**Implementation Techniques:**

* **Abstract Classes:** Classes that cannot be instantiated directly but provide a blueprint for concrete subclasses.
* **Interfaces:** Contracts that define methods without implementation. Classes can implement interfaces to enforce specific behaviors.

**Benefits:**

* **Complexity Management:** Simplifies interaction with complex systems.
* **Code Flexibility:** Allows for internal changes without affecting the user.
* **Increased Modularity:** Promotes loose coupling between components.

**Example (Abstract Class):**

```java
abstract class Vehicle { // Abstract class
    public abstract void start(); // Abstract method
    
    public void stop() {
        System.out.println("Vehicle stopped.");
    }
}

class Car extends Vehicle {
    @Override
    public void start() {
        System.out.println("Car started.");
    }
}
```

**Explanation:** The `Vehicle` class is abstract, so you can't create instances of it directly. The `start()` method is abstract and needs to be implemented by concrete subclasses like `Car`.

### References

* **Head First Java** by Kathy Sierra and Bert Bates
* **Effective Java** by Joshua Bloch
* **Oracle Java Documentation:** [https://docs.oracle.com/javase/tutorial/](https://docs.oracle.com/javase/tutorial/)

This presentation provides a foundation for understanding the four pillars of OOP in Java. By mastering these principles, you'll write cleaner, more efficient, and maintainable code. 
