# Generics in Java

## Introduction

Generics, introduced in Java 5, are a powerful feature that allows you to write type-safe code that can work with various data types without requiring explicit casting. They add compile-time type safety and eliminate the risk of `ClassCastException` at runtime. Before generics, collections like `ArrayList` stored objects as `Object`, requiring casts when retrieving elements. Generics enable specifying the type of objects a collection can hold, enhancing type safety and readability.

## Why Generics?

* **Type Safety:** Compile-time checks ensure only compatible types are used.
* **Code Reusability:** Write algorithms that work with different types without modification.
* **Elimination of Casts:** No more explicit casting, improving code clarity.
* **Improved Performance:** Type information available at compile time can lead to optimizations.

## Generic Classes

A generic class is defined with type parameters enclosed in angle brackets `< >`. These parameters act as placeholders for concrete types when the class is used.

```java
public class Box<T> {
    private T value;

    public Box(T value) {
        this.value = value;
    }

    public T getValue() {
        return value;
    }
}
```

Here, `T` is a type parameter.  When instantiating `Box`, we provide the actual type:

```java
Box<Integer> intBox = new Box<>(10);
Box<String> stringBox = new Box<>("Hello");

int intValue = intBox.getValue(); // No cast needed!
String stringValue = stringBox.getValue();
```

## Generic Methods

Generic methods can be defined within both generic and non-generic classes.  They have their own type parameters, declared before the return type.

```java
public class Utils {
    public static <T> T identity(T value) {
        return value;
    }

    public static <K, V> boolean compareKeys(Pair<K, V> p1, Pair<K, V> p2) {
        return p1.getKey().equals(p2.getKey());
    }
}

// Usage
Integer num = Utils.identity(5);
String str = Utils.identity("Java");
```

## Bounded Type Parameters

Restrict the types that can be used with a generic type parameter using extends or super.

* **Upper Bounded Wildcards:** `<? extends T>` -  Allows types that are subtypes of `T` or `T` itself.  Useful for reading elements from a collection.

```java
List<? extends Number> numbers = new ArrayList<Integer>(); // Allowed
numbers.add(5); // Compile error - can't add anything
Number n = numbers.get(0); // Allowed
```

* **Lower Bounded Wildcards:** `<? super T>` - Allows types that are supertypes of `T` or `T` itself. Useful for adding elements to a collection.

```java
List<? super Integer> ints = new ArrayList<Number>(); // Allowed
ints.add(5); // Allowed
Number n = ints.get(0); // Compile error - could be an Object
```


## Wildcards

Wildcards (`?`) are used when the specific type isn't relevant but you still want type safety.

```java
public void printList(List<?> list) {
    for (Object item : list) {
        System.out.println(item);
    }
}
```

## Type Erasure

Generics in Java are implemented using type erasure. At compile time, the compiler replaces type parameters with their upper bounds (or `Object` if unbounded) and inserts casts where necessary. This means that at runtime, generic type information is not fully available.


##  Common Pitfalls

* Cannot create instances of type parameters (`new T()`).
* Type parameters cannot be used in static contexts.
* Issues with overloading and overriding methods with generics.


## Further Reading

* **The Java Tutorials - Generics:** [https://docs.oracle.com/javase/tutorial/java/generics/index.html](https://docs.oracle.com/javase/tutorial/java/generics/index.html)
* **Effective Java (3rd Edition) by Joshua Bloch:** Covers generics in detail, including best practices and common pitfalls.
* **Java Generics and Collections by Maurice Naftalin and Philip Wadler:** A comprehensive guide to generics and their use with collections.


## Conclusion

Generics are a crucial part of modern Java development, promoting type safety, code reusability, and improved performance. Understanding generics, including bounded type parameters, wildcards, and type erasure, is essential for writing robust and maintainable Java applications.  This presentation provides a foundational overview, and the provided resources offer deeper dives into specific aspects of this powerful feature.
