## Type Erasure in Java: Losing Type Information at Runtime

**Slide 1: Title Slide**

* **Title:** Type Erasure in Java: The Mystery of Vanishing Generics
* **Subtitle:** Understanding the Runtime Behavior of Generics
* **Your Name & Affiliation**
* **Course Name & Number**

**Slide 2: Introduction to Generics**

* **Motivation:** Why were generics introduced in Java 5?
    * Code reusability without compromising type safety.
    * Eliminate ClassCastException at runtime by shifting type checking to compile time.
* **Example (Pre-Generics):**
```java
List list = new ArrayList();
list.add("Hello");
list.add(123); // No compile-time error

String str = (String) list.get(1); // Runtime ClassCastException
```
* **Example (With Generics):**
```java
List<String> list = new ArrayList<>();
list.add("Hello");
// list.add(123); // Compile-time error: incompatible types
String str = list.get(0); // No casting required
```

**Slide 3: The Concept of Type Erasure**

* **Definition:** Type erasure is a process by which the Java compiler removes generic type information at compile time.  This means that at runtime, the JVM doesn't know the specific type parameters used with a generic class or method.
* **Reason:** Backwards compatibility with pre-Java 5 code.  Generics were designed to be a source-code level feature without requiring changes to the JVM's bytecode representation.
* **Consequence:**  Generic type information is not available at runtime.  This has implications for reflection and some other operations.

**Slide 4: How Type Erasure Works**

* **Raw Types:** During compilation, the compiler replaces generic types with their raw types (the non-generic version of the class).  `List<String>` becomes `List`.
* **Type Casting:** For parameterized types, the compiler inserts casts where necessary to ensure type safety at runtime. This is why you don't need to explicitly cast when retrieving elements from a generic collection.
* **Bridge Methods:** To maintain polymorphism with generics, the compiler might generate synthetic bridge methods. These methods handle the translation between generic and raw types, particularly when overriding methods from a non-generic superclass.


**Slide 5: Example of Bridge Methods**

```java
class MyList<T> {
    public void add(T element) {}
}

class StringList extends MyList<String> {
    @Override
    public void add(String element) {} // This overrides the generic method

    // Compiler generates a bridge method:
    // public void add(Object element) { add((String) element); }
}
```

**Slide 6: Implications of Type Erasure**

* **Runtime Type Information:** `instanceof` checks and `getClass()` calls on generic types operate on the erased type.
    ```java
List<String> stringList = new ArrayList<>();
System.out.println(stringList instanceof List<String>); // Compiler warning, evaluates to true at runtime (instanceof List)
System.out.println(stringList.getClass()); // Output: class java.util.ArrayList
```
* **Generic Array Creation:** You cannot create arrays of generic types directly due to type erasure. Workarounds include using `ArrayList` or creating an array of the raw type and casting (with a warning).
* **Type Parameter Restrictions:**  Certain operations are restricted on type parameters due to erasure. For example, you cannot create instances of type parameters (`new T()`), or use them in `switch` statements.

**Slide 7: Workarounds and Best Practices**

* **Use wildcard types (`? extends T`, `? super T`)** to increase flexibility when dealing with unknown types.
* **Favor type inference (diamond operator `<>`)** to reduce code verbosity.
* **Understand the limitations of reflection with generics.** Use `getGenericType()` methods to access erased type information.

**Slide 8: References and Further Reading**

* **The Java Tutorials: Generics:** [https://docs.oracle.com/javase/tutorial/java/generics/](https://docs.oracle.com/javase/tutorial/java/generics/)
* **Effective Java (3rd Edition) by Joshua Bloch:** Chapter 5 (Generics) provides in-depth discussion on type erasure and its implications.
* **Java Generics in Depth by Maurice Naftalin and Philip Wadler:** A comprehensive book covering advanced aspects of generics.


**Slide 9: Q&A**


This Markdown can be converted to slides using various tools, like Pandoc or online Markdown presentation creators.  Remember to adapt the content and examples based on your students' level and the course objectives. Good luck!
