## Concurrency in Java

This course material explores concurrency in Java, covering fundamental concepts, common problems, and best practices.

**1. Introduction to Concurrency**

Concurrency allows multiple tasks to progress seemingly simultaneously. In Java, this is achieved through threads.  A thread is a lightweight unit of execution within a process.  Concurrency enhances performance by utilizing multiple cores and overlapping I/O operations.

**2. Threads and Processes**

| Feature | Process | Thread |
|---|---|---|
| Memory Space | Own independent memory space | Shares memory space with other threads within the same process |
| Creation | Resource intensive | Less resource intensive |
| Communication | Inter-process communication mechanisms required | Shared memory facilitates easier communication |
| Context Switching | More overhead | Less overhead |


```
+-----------------+     +-----------------+
|     Process 1   |     |     Process 2   |
| +-----------+ |     | +-----------+ |
| | Thread 1  | |     | | Thread 1  | |
| +-----------+ |     | +-----------+ |
| | Thread 2  | |     | | Thread 2  | |
| +-----------+ |     | +-----------+ |
+-----------------+     +-----------------+
```

**3. Creating and Managing Threads**

Two primary ways to create threads:

* **Extending the `Thread` class:**

```java
class MyThread extends Thread {
    @Override
    public void run() {
        System.out.println("Thread " + Thread.currentThread().getName() + " is running.");
    }
}

public class Main {
    public static void main(String[] args) {
        MyThread thread1 = new MyThread();
        thread1.start(); // Starts the thread
    }
}
```

* **Implementing the `Runnable` interface:**

```java
class MyRunnable implements Runnable {
    @Override
    public void run() {
        System.out.println("Thread " + Thread.currentThread().getName() + " is running.");
    }
}

public class Main {
    public static void main(String[] args) {
        Thread thread2 = new Thread(new MyRunnable());
        thread2.start();
    }
}
```


**4. Synchronization and Thread Safety**

Concurrent access to shared resources can lead to race conditions and data inconsistencies.  Synchronization mechanisms are essential to ensure thread safety.

* **`synchronized` keyword:**

```java
public class Counter {
    private int count = 0;

    public synchronized void increment() {
        count++;
    }
}
```

* **Locks (`java.util.concurrent.locks`):**  Provide more flexible locking mechanisms like `ReentrantLock` and `ReadWriteLock`.

```java
import java.util.concurrent.locks.ReentrantLock;

public class Counter {
    private int count = 0;
    private ReentrantLock lock = new ReentrantLock();

    public void increment() {
        lock.lock();
        try {
            count++;
        } finally {
            lock.unlock();
        }
    }
}
```

**5. Common Concurrency Problems**

* **Race Conditions:** Multiple threads accessing and modifying shared data simultaneously, leading to unpredictable results.
* **Deadlocks:** Two or more threads are blocked indefinitely, waiting for each other to release resources.
* **Starvation:** A thread is perpetually denied access to resources it needs.
* **Livelock:**  Threads are constantly changing their state in response to each other, but no progress is made.


**6. Concurrent Collections**

Java provides concurrent collections like `ConcurrentHashMap`, `ConcurrentLinkedQueue`, etc., designed for thread-safe operations without explicit locking.

**7. Executors and Thread Pools**

The `java.util.concurrent.ExecutorService` interface provides a higher-level abstraction for managing threads.  Thread pools allow reusing threads, reducing overhead.

```java
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class Main {
    public static void main(String[] args) {
        ExecutorService executor = Executors.newFixedThreadPool(5);
        executor.execute(new MyRunnable());
        executor.shutdown();
    }
}

```

**8. Atomic Variables**

Classes like `AtomicInteger`, `AtomicLong` provide atomic operations, ensuring that updates to variables happen as a single, indivisible action.


**References:**

* **Book:** "Java Concurrency in Practice" by Brian Goetz et al.
* **Article:**  "The Java™ Tutorials: Concurrency" (Oracle Documentation) - [https://docs.oracle.com/javase/tutorial/essential/concurrency/](https://docs.oracle.com/javase/tutorial/essential/concurrency/)
* **Documentation:** Java API documentation for `java.util.concurrent`


This course material provides a foundation for understanding concurrency in Java.  Further exploration of advanced topics like Fork/Join framework, asynchronous programming, and specific concurrency patterns is encouraged.
