## Concurrency Models in Java

This course material explores the different concurrency models available in Java, allowing developers to write efficient and thread-safe programs.  We will delve into various approaches, including threads, the Executor framework, and concurrent data structures.

**1. Threads: The Foundation of Concurrency**

At the core of Java's concurrency model lies the `Thread` class.  Each thread represents an independent flow of execution within a program.

* **Creating Threads:**

  * **Extending the `Thread` class:**

    ```java
    class MyThread extends Thread {
        @Override
        public void run() {
            // Code to be executed in the new thread
            System.out.println("Running in a separate thread.");
        }
    }

    // Usage:
    MyThread myThread = new MyThread();
    myThread.start(); // Starts the thread
    ```

  * **Implementing the `Runnable` interface:** (Preferred due to flexibility with inheritance)

    ```java
    class MyRunnable implements Runnable {
        @Override
        public void run() {
            System.out.println("Running in a separate thread via Runnable.");
        }
    }

    // Usage:
    Thread thread = new Thread(new MyRunnable());
    thread.start();
    ```

* **Thread Lifecycle:**

```
     NEW          -> RUNNABLE <-> RUNNING
       |                 ^        |
       |                 |        v
       +----TERMINATED<---+     BLOCKED/WAITING/TIMED_WAITING
```


* **Key Thread Methods:**

    * `start()`:  Starts the thread's execution.
    * `run()`: Contains the code to be executed by the thread.
    * `join()`: Waits for the thread to complete.
    * `sleep()`: Pauses the thread for a specified duration.
    * `interrupt()`: Interrupts the thread's execution (e.g., when blocked).


**2. Synchronization and Thread Safety**

Multiple threads accessing shared resources can lead to data inconsistencies and race conditions.  Synchronization mechanisms ensure thread safety.

* **`synchronized` keyword:**

    * **Method synchronization:**  Synchronizes the entire method. Only one thread can execute a synchronized method on an object at a time.

    ```java
    public synchronized void updateCounter() {
        // Thread-safe code here
    }
    ```

    * **Block synchronization:** Synchronizes a specific block of code within a method.

    ```java
    public void updateCounter() {
        synchronized (this) { // 'this' refers to the current object
            // Thread-safe code here
        }
    }
    ```

* **`volatile` keyword:**  Ensures visibility of changes to a variable across threads. Prevents caching of variable values in thread-local memory.

* **`java.util.concurrent.locks` package:** Provides more sophisticated locking mechanisms like `ReentrantLock`, `ReadWriteLock`, etc.

**3. Executor Framework**

The Executor framework simplifies thread management and provides a higher-level abstraction for concurrency.

* **`ExecutorService`:**  An interface for managing a pool of threads.

* **`Executors`:** A factory class for creating various types of `ExecutorService` implementations (e.g., `Executors.newFixedThreadPool(int nThreads)`).

* **`Future`:** Represents the result of an asynchronous computation.

```java
ExecutorService executor = Executors.newFixedThreadPool(5);
Future<Integer> future = executor.submit(new Callable<Integer>() {
    @Override
    public Integer call() throws Exception {
        // Perform some computation
        return 42;
    }
});

// ... do other work ...

int result = future.get(); // Blocks until the result is available
executor.shutdown(); // Shut down the executor
```



**4. Concurrent Data Structures**

Java provides thread-safe implementations of common data structures in the `java.util.concurrent` package. Examples include:

* `ConcurrentHashMap`
* `ConcurrentLinkedQueue`
* `BlockingQueue`
* `AtomicInteger`, `AtomicLong`, etc. (for atomic operations)


**5. Further Reading:**

* **Books:**
    * "Java Concurrency in Practice" by Brian Goetz et al. (Highly recommended)
    * "Effective Java" by Joshua Bloch (Covers concurrency best practices)

* **Articles:**
    * [Various articles on Oracle's Java Concurrency documentation](https://docs.oracle.com/en/java/javase/17/docs/api/java.base/java/util/concurrent/package-summary.html)


This material provides a starting point for understanding concurrency models in Java.  Hands-on practice and deeper exploration of the referenced materials are crucial for mastering concurrent programming. Remember to consider performance implications and choose the appropriate concurrency model for your specific needs.
