## Runnable Interface in Java

The `Runnable` interface in Java is a core part of the concurrency framework and provides a way to define a task that can be executed by a separate thread. It's a functional interface, meaning it has only one abstract method: `run()`. This method contains the code that will be executed when the thread starts.

**Why use Runnable?**

* **Separation of Concerns:** `Runnable` allows you to separate the task logic from the thread management mechanism.  You define *what* needs to be done in the `run()` method, while the `Thread` class handles *how* it's executed.
* **Flexibility:** Implementing `Runnable` provides more flexibility than extending the `Thread` class, especially if your class already inherits from another class (Java doesn't support multiple inheritance).
* **Code Reusability:** A `Runnable` object can be reused across multiple threads, making it more efficient than creating a new thread for each task.

**How to use Runnable:**

1. **Implement the Runnable Interface:** Create a class that implements the `Runnable` interface and override the `run()` method.

   ```java
   class MyRunnable implements Runnable {
       @Override
       public void run() {
           // Code to be executed by the thread
           System.out.println("Executing in a separate thread: " + Thread.currentThread().getName());
       }
   }
   ```

2. **Create a Thread object:** Instantiate a `Thread` object, passing an instance of your `Runnable` implementation to its constructor.

   ```java
   MyRunnable runnable = new MyRunnable();
   Thread thread = new Thread(runnable);
   ```

3. **Start the thread:** Call the `start()` method on the `Thread` object. This will create a new thread of execution and invoke the `run()` method of the provided `Runnable` object.

   ```java
   thread.start(); 
   ```

**Example: Implementing Runnable with Lambda Expressions (Java 8 and later)**

Lambda expressions offer a concise way to implement functional interfaces like `Runnable`:

```java
Runnable runnable = () -> {
    System.out.println("Executing using a lambda expression: " + Thread.currentThread().getName());
};

Thread thread = new Thread(runnable);
thread.start();
```

**Example: Handling InterruptedException:**

The `run()` method can throw an `InterruptedException`.  This typically occurs when another thread interrupts this thread while it's sleeping or waiting. It's good practice to handle this exception.

```java
class MyRunnable implements Runnable {
    @Override
    public void run() {
        try {
            Thread.sleep(2000); // Simulate some work
            System.out.println("Thread completed.");
        } catch (InterruptedException e) {
            System.out.println("Thread interrupted.");
            Thread.currentThread().interrupt(); // Preserve the interrupt status
        }
    }
}
```

**Key Considerations:**

* **Thread Safety:** When multiple threads access shared resources, you need to implement appropriate synchronization mechanisms (e.g., locks, synchronized blocks) to prevent data corruption or race conditions.
* **Thread Lifecycle:** Understand the different states of a thread (New, Runnable, Blocked, Waiting, Timed_Waiting, Terminated) and how they transition between these states.
* **Thread Pools:** For managing multiple threads efficiently, consider using the `ExecutorService` interface and thread pools from the `java.util.concurrent` package.


**References:**

* **Documentation:** [Oracle Java Documentation on Runnable](https://docs.oracle.com/en/java/javase/17/docs/api/java.base/java/lang/Runnable.html)
* **Books:**
    * "Effective Java" by Joshua Bloch (Covers best practices for multithreading)
    * "Java Concurrency in Practice" by Brian Goetz et al. (A comprehensive guide to concurrent programming in Java)
* **Articles:**
    * [Baeldung: Introduction to the Runnable Interface](https://www.baeldung.com/java-runnable)
    * [Oracle Tutorials: Concurrency](https://docs.oracle.com/javase/tutorial/essential/concurrency/)



This material provides a solid foundation for understanding and using the `Runnable` interface in Java. Students should be encouraged to experiment with the code examples and explore the provided references for a deeper understanding of concurrency. They should also be reminded of the importance of thread safety and proper resource management in multithreaded applications.
