## Non-Blocking I/O (NIO) in Java

This course material introduces Java NIO, a powerful API offering high-scalability for network and file operations through its non-blocking capabilities.

### 1. Introduction to NIO

Traditional I/O in Java uses blocking operations. When a thread reads from a stream or writes to a stream, the thread blocks until the operation completes. This can be inefficient, particularly in scenarios with high concurrency.

NIO addresses this limitation by providing non-blocking I/O operations.  A thread can initiate an I/O operation and continue working on other tasks.  When the I/O operation completes, the thread is notified. This allows a single thread to manage multiple connections simultaneously, improving performance and scalability.

**Key Components of NIO:**

* **Channels:** Represent open connections to I/O resources (files, sockets, etc.). Similar to streams but bi-directional.
* **Buffers:**  Containers for data being transferred between channels. Data is read into a buffer from a channel or written from a buffer to a channel.
* **Selectors:** Objects that monitor multiple channels for events (readiness for read/write operations). Allows a single thread to handle multiple channels efficiently.


### 2. Channels

Channels are the conduits for data transfer in NIO.  Common channel types include:

* `SocketChannel`:  For TCP connections.
* `ServerSocketChannel`: For listening for incoming TCP connections.
* `DatagramChannel`: For UDP connections.
* `FileChannel`: For file I/O.

**Example: Reading from a FileChannel:**

```java
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;

public class FileChannelExample {

    public static void main(String[] args) throws IOException {
        Path filePath = Paths.get("my_file.txt");
        FileChannel fileChannel = FileChannel.open(filePath, StandardOpenOption.READ);

        ByteBuffer buffer = ByteBuffer.allocate(1024); // Allocate a buffer

        while (fileChannel.read(buffer) != -1) { // Read data into buffer
            buffer.flip(); // Prepare buffer for reading

            while (buffer.hasRemaining()) {
                System.out.print((char) buffer.get());
            }

            buffer.clear(); // Clear buffer for next read
        }

        fileChannel.close();
    }
}

```

### 3. Buffers

Buffers are containers for data. Key buffer operations include:

* `allocate()`: Creates a buffer of a specified size.
* `put()`: Writes data to the buffer.
* `get()`: Reads data from the buffer.
* `flip()`: Switches the buffer from write mode to read mode.
* `clear()`: Clears the buffer.

### 4. Selectors

Selectors allow a single thread to manage multiple channels.  They monitor registered channels for events like read readiness and write readiness.

**Example: A Simple Echo Server Using NIO:**

```java
//  (Code for a basic echo server using Selector, ServerSocketChannel, and SocketChannel would go here.  This is a more complex example and requires handling selection keys, registering channels with selectors, and processing read/write events).

// ... (Code omitted for brevity – see references below for complete examples)
```

### 5.  Advanced Topics

* **Scatter/Gather I/O:**  Reading or writing data from/to multiple buffers in a single operation.
* **Memory-Mapped Files:** Mapping a file to memory for efficient access.
* **Non-blocking asynchronous operations.**


### References

**Books:**

* **Java NIO, 2nd Edition by Ron Hitchens:** A comprehensive guide to NIO.
* **Java Network Programming, 4th Edition by Elliotte Rusty Harold:** Covers NIO in the context of network programming.


**Articles/Documentation:**

* **Oracle's Java NIO Tutorial:**  [https://docs.oracle.com/javase/tutorial/essential/io/nio.html](https://docs.oracle.com/javase/tutorial/essential/io/nio.html)
* **Various articles and blogs available online (search for "Java NIO Tutorial" or "Java NIO Examples").**


**Exercises:**

1. Modify the `FileChannelExample` to write data to a file.
2. Implement a simple client that connects to the echo server (code provided separately or as an exercise).
3. Explore Scatter/Gather I/O and implement a simple example.


This material provides a starting point for learning Java NIO. The examples and references provided will help you delve deeper into the subject and build sophisticated, scalable applications.  Remember to practice by writing your own NIO code to solidify your understanding.
