# Java Course Material: Understanding Method Overriding and Overloading

## Table of Contents
1. Introduction to Method Overriding and Overloading
2. Detailed Explanation of Method Overriding
   - Definition
   - Inheritance and Polymorphism
   - Code Examples
   - Annotations in Overriding
3. Detailed Explanation of Method Overloading
   - Definition
   - Compile-Time Polymorphism
   - Code Examples
4. Differences and Similarities between Method Overriding and Overloading
   - Comparison
   - Code Examples
5. References

---

## 1. Introduction to Method Overriding and Overloading

Java supports two techniques to implement polymorphism: **method overriding** and **method overloading**. While both allow a class to define multiple methods, they achieve different outcomes and purposes within the Java language:

### Method Overriding
- Allows a subclass to provide a specific implementation of a method that is already defined in its superclass.

### Method Overloading
- Allows a class to have multiple methods with the same name but different parameters (different type, number, or both).

---

## 2. Detailed Explanation of Method Overriding

### Definition
Method overriding occurs when a subclass declares a method with the **same name, return type, and parameters** as a method in its superclass. It enables runtime polymorphism and enhances the functionality of a class.

### Inheritance and Polymorphism
Method overriding is a key feature of **inheritance** in Java. It is used to achieve **polymorphism**, where the correct overridden method is called at runtime on the basis of the object type.

### Code Examples

```java
class Animal {
    void sound() {
        System.out.println("Animal makes a sound");
    }
}

class Dog extends Animal {
    @Override
    void sound() {
        System.out.println("Dog barks");
    }
}

public class Test {
    public static void main(String[] args) {
        Animal obj = new Dog();
        obj.sound(); // Outputs: Dog barks
    }
}
```

### Annotations in Overriding

#### Importance of @Override

- **@Override** is an annotation used to indicate that a method is intended to override a method in a superclass.
- It helps catch errors at compile time (e.g., if the method signature is not correctly overridden).

#### Code Example with @Override

```java
class Vehicle {
    void run() {
        System.out.println("Vehicle is running");
    }
}

class Bike extends Vehicle {
    @Override
    void run() {
        System.out.println("Bike is running safely");
    }
}
```

#### Implications of Not Using @Override

- Without **@Override**, if the method signature is incorrectly overridden due to typos or wrong parameters, the program may compile without warnings or errors, but won't achieve the desired polymorphic behavior.

References:
- [Java Documentation on Annotations](https://docs.oracle.com/javase/tutorial/java/annotations/)
- **Effective Java** by Joshua Bloch 

---

## 3. Detailed Explanation of Method Overloading

### Definition

Method overloading occurs within the same class to provide more than one method with the same name but different parameters. It is related to compile-time (static) polymorphism.

### Code Examples

```java
class MathUtils {
    int sum(int a, int b) {
        return a + b;
    }

    double sum(double a, double b) {
        return a + b;
    }

    int sum(int a, int b, int c) {
        return a + b + c;
    }
}

public class Test {
    public static void main(String[] args) {
        MathUtils mathUtils = new MathUtils();
        System.out.println(mathUtils.sum(10, 20)); // Calls the first sum method
        System.out.println(mathUtils.sum(10.5, 20.5)); // Calls the second sum method
        System.out.println(mathUtils.sum(10, 20, 30)); // Calls the third sum method
    }
}
```

---

## 4. Differences and Similarities between Method Overriding and Overloading

### Differences

| Feature                | Method Overloading                                 | Method Overriding                                         |
|------------------------|----------------------------------------------------|-----------------------------------------------------------|
| Definition             | Same method name, different parameters             | Same method name, same parameters in a subclass           |
| Purpose                | To increase readability and flexibility            | To provide specific implementation in a subclass          |
| Polymorphism           | Compile-time (static)                              | Runtime (dynamic)                                         |
| Annotation             | Not required                                       | @Override is recommended                                  |

### Similarities

- Both allow programmers to create multiple methods with the same name.
- Both are used to implement polymorphism (compile-time and runtime).
- Both improve code readability and maintainability.

### Code Comparison Example

```java
class Base {
    void display() {
        System.out.println("Base display");
    }
}

class Derived extends Base {
    @Override
    void display() {
        System.out.println("Derived display");
    }
    
    // Overloaded method
    void display(String message) {
        System.out.println("Display Message: " + message);
    }
}

public class Test {
    public static void main(String[] args) {
        Base obj = new Derived();
        obj.display(); // Calls Derived class method (override)

        Derived derivedObj = new Derived();
        derivedObj.display("Hello"); // Calls overloaded method
    }
}
```

---

## 5. References

- **Effective Java**, Joshua Bloch, 3rd Edition.
- Oracle Java Documentation: [Method Overriding and Overloading](https://docs.oracle.com/javase/tutorial/java/IandI/override.html)
- [Java Annotations Tutorial](https://docs.oracle.com/javase/tutorial/java/annotations/)
- [Java Platform Standard Edition Documentation](https://docs.oracle.com/javase/8/docs/api/)
