# Course Material: Test Driven Development with JUnit in Java

## Slide 1: Introduction to Test Driven Development (TDD)

**TDD Overview**

- **Definition**: Test-Driven Development (TDD) is a software development approach where tests are written before writing the bare minimum of code required for the test to be fulfilled.
- **Principles**:
  - Write a test for a new feature or function.
  - Run all tests and see if the new one fails (the test should not pass immediately).
  - Write the code to make the test pass.
  - Run all tests again.
  - Refactor the code for optimization.
- **Benefits**:
  - Reduces the number of bugs in production.
  - Ensures the software design is cleaner and more flexible.
  - Helps programmers understand the problem domain.
  
**References**:
- Beck, K. (2002). *Test Driven Development: By Example*. Addison-Wesley.
- Astels, D. (2003). *Test-Driven Development: A Practical Guide*. Prentice Hall.

## Slide 2: Introduction to JUnit

**What is JUnit?**

- **Description**: JUnit is a popular framework used for writing and running unit tests in Java. It provides annotations to identify test methods and contains numerous assertion methods for checking expected outcomes.

**Key Features**:
- Provides simple and efficient testing.
- Offers annotations such as `@Test`, `@BeforeEach`, `@AfterEach`, `@BeforeAll`, `@AfterAll`.
- Supports test suites for bundle and run multiple test cases.
  
**References**:
- Gamma, E. & Beck, K. *JUnit Cookbook*.

## Slide 3: Types of Unit Tests with JUnit

**Types of JUnit Tests**

1. **Standard Unit Tests**
   - Focus on small functionality.
   - Quick execution.
   
   **Example**:
   ```java
   import static org.junit.jupiter.api.Assertions.assertEquals;
   import org.junit.jupiter.api.Test;

   public class CalculatorTest {
       @Test
       public void testAdd() {
           Calculator calculator = new Calculator();
           int result = calculator.add(10, 20);
           assertEquals(30, result);
       }
   }
   ```

2. **Boundary Tests**
   - Test limits or boundaries of variables.
   
   **Example**:
   ```java
   import static org.junit.jupiter.api.Assertions.assertThrows;

   public class StringTest {
       @Test
       public void emptyStringLength() {
           String str = "";
           assertEquals(0, str.length());
       }
   }
   ```

3. **Exception Tests**
   - Ensure correct exceptions are thrown.

   **Example**:
   ```java
   @Test
   public void testForException() {
       assertThrows(ArithmeticException.class, () -> {
           int division = 100 / 0;
       });
   }
   ```

**References**:
- Meszaros, G. (2007). *xUnit Test Patterns: Refactoring Test Code*. Addison-Wesley.

## Slide 4: Parameterized Tests in JUnit

**Overview of Parameterized Tests**

- **Definition**: Parameterized tests are a way to run the same test with different data sets.
- **Purpose**: Reduce code duplication in tests and improve test maintenance.

**Key Components**:
- Use `@ParameterizedTest` annotation.
- Define data sources such as CSV files, Java Streams with `@CsvSource`, and more.

**Example**:
```java
import static org.junit.jupiter.api.Assertions.assertTrue;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;

public class NumberTest {
    @ParameterizedTest
    @ValueSource(ints = { 2, 4, 6, 8, 10 })
    void isEven(int number) {
        assertTrue(number % 2 == 0);
    }
}
```

**References**:
- Hardcore Java Unit Testing Coverage with @Parameterize. *Documentation and Tutorials*.

## Slide 5: Setting up JUnit in Your Java Project

**Steps to Setup**:
1. **Use a Build Tool**: Use Maven/Gradle to manage JUnit dependencies.
2. **Add Dependencies**: Add the following dependency to your `pom.xml` for Maven.
   
   ```xml
   <dependency>
       <groupId>org.junit.jupiter</groupId>
       <artifactId>junit-jupiter</artifactId>
       <version>5.7.0</version>
       <scope>test</scope>
   </dependency>
   ```

3. **Configure Classpath**: Ensure `junit-jupiter` is on the classpath.

## Slide 6: Conclusion

**Summary**

- Test-driven development and unit testing with JUnit lead to higher quality code.
- Understanding different test types enables more thorough and meaningful tests.
- Parameterized tests offer flexibility and efficient test coverage.

**Key Takeaways**

- Start with writing a failing test.
- Iterate code to make it pass, then refactor.
- Use JUnit for structure and ease of testing.

**Next Steps**
- Explore JUnit official documentation for a deeper dive into advanced features.
- Experiment by writing JUnit tests for personal projects.

**References**
- The JUnit Team. (Current Version Documentation). JUnit 5: User Guide. Accessible at: [JUnit 5 User Guide](https://junit.org/junit5/docs/current/user-guide/)

--- 

This format serves as a structured presentation on TDD using JUnit in Java, encompassing fundamental aspects, practical coding examples, and resource references for further exploration.