## Java Control Flow and Object Manipulation

Welcome to this comprehensive guide on Java control flow statements and working with objects. This presentation covers:

* **Control Flow Statements**:
    * if-else
    * for loops
    * for-each loops
    * while loops
    * do-while loops
    * switch statements
    * switch expressions (Java 14+)
    * try-catch
    * try-with-resources
* **Object Interaction**:
    * Invoking object and class methods
    * Accessing object and class fields
* **Iterables and Iterators**

Let's dive in!

---

### Control Flow Statements

Control flow statements dictate the order in which code is executed. They enable decision-making, looping, and exception handling.

#### 1. if-else Statements

The `if-else` statement executes different blocks of code based on a condition.

**Syntax:**

```java
if (condition) {
  // Code executed if condition is true
} else if (anotherCondition) {
  // Code executed if anotherCondition is true
} else {
  // Code executed if no previous conditions are true
}
```

**Example:**

```java
int age = 25;

if (age >= 18) {
  System.out.println("You are an adult.");
} else {
  System.out.println("You are a minor.");
}
```

**Reference:** [Oracle Docs - The if-then-else Statement](https://docs.oracle.com/javase/tutorial/java/nutsandbolts/if.html)

#### 2. for Loops

`for` loops execute a block of code repeatedly for a specified number of times.

**Syntax:**

```java
for (initialization; condition; increment/decrement) {
  // Code to be executed repeatedly
}
```

**Example:**

```java
for (int i = 0; i < 5; i++) {
  System.out.println("Iteration: " + i);
}
```

**Reference:** [Oracle Docs - The for Statement](https://docs.oracle.com/javase/tutorial/java/nutsandbolts/for.html)

#### 3. for-each Loops

`for-each` loops iterate over each element in an array or collection.

**Syntax:**

```java
for (dataType element : array/collection) {
  // Code to be executed for each element
}
```

**Example:**

```java
String[] fruits = {"Apple", "Banana", "Orange"};
for (String fruit : fruits) {
  System.out.println(fruit);
}
```

**Reference:** [Oracle Docs - The Enhanced for Loop](https://docs.oracle.com/javase/tutorial/java/nutsandbolts/for.html)

#### 4. while Loops

`while` loops execute a block of code as long as a condition remains true.

**Syntax:**

```java
while (condition) {
  // Code to be executed repeatedly
}
```

**Example:**

```java
int count = 0;
while (count < 10) {
  System.out.println("Count: " + count);
  count++;
}
```

**Reference:** [Oracle Docs - The while and do-while Statements](https://docs.oracle.com/javase/tutorial/java/nutsandbolts/while.html)

#### 5. do-while Loops

`do-while` loops are similar to `while` loops but guarantee at least one execution of the code block.

**Syntax:**

```java
do {
  // Code to be executed repeatedly
} while (condition);
```

**Example:**

```java
int number = 5;
do {
  System.out.println("Number: " + number);
  number--;
} while (number > 0);
```

**Reference:** [Oracle Docs - The while and do-while Statements](https://docs.oracle.com/javase/tutorial/java/nutsandbolts/while.html)

#### 6. switch Statements

`switch` statements allow multi-way branching based on the value of an expression.

**Syntax:**

```java
switch (expression) {
  case value1:
    // Code executed if expression matches value1
    break;
  case value2:
    // Code executed if expression matches value2
    break;
  // ... more cases
  default:
    // Code executed if no other case matches
}
```

**Example:**

```java
int day = 3;
switch (day) {
  case 1:
    System.out.println("Monday");
    break;
  case 2:
    System.out.println("Tuesday");
    break;
  // ... more cases
  default:
    System.out.println("Invalid day");
}
```

**Reference:** [Oracle Docs - The switch Statement](https://docs.oracle.com/javase/tutorial/java/nutsandbolts/switch.html)


#### 7. Switch Statements with Pattern Matching (Java 14+)

Java 14 introduced pattern matching for `switch` statements, allowing for more concise and expressive code.

**Syntax:**

```java
switch (object) {
  case Type1 pattern1 -> {
    // Code for Type1 pattern1 match
  }
  case Type2 pattern2 -> {
    // Code for Type2 pattern2 match
  }
  // ... more cases
  default -> {
    // Code for no match
  }
}
```

**Example:**

```java
Object obj = "Hello";
switch (obj) {
  case String s -> System.out.println("String: " + s);
  case Integer i -> System.out.println("Integer: " + i);
  default -> System.out.println("Unknown type");
}
```

**Reference:** [JEP 361: Switch Expressions](https://openjdk.java.net/jeps/361)


#### 8. try-catch Statements

`try-catch` statements handle exceptions, which are unexpected events that occur during program execution.

**Syntax:**

```java
try {
  // Code that might throw an exception
} catch (ExceptionType1 e1) {
  // Code to handle ExceptionType1
} catch (ExceptionType2 e2) {
  // Code to handle ExceptionType2
} finally {
  // Code always executed, whether an exception occurred or not
}
```

**Example:**

```java
try {
  int result = 10 / 0;
} catch (ArithmeticException e) {
  System.out.println("Error: Division by zero.");
}
```

**Reference:** [Oracle Docs - Exceptions](https://docs.oracle.com/javase/tutorial/essential/exceptions/index.html)


#### 9. try-with-resources Statements

`try-with-resources` statements provide a cleaner way to manage resources that need to be closed after use (e.g., files, database connections).

**Syntax:**

```java
try (ResourceType resource = ...; ResourceType resource2 = ...) {
  // Code using the resources
} catch (Exception e) {
  // Code to handle exceptions
}
```

**Example:**

```java
try (BufferedReader reader = new BufferedReader(new FileReader("file.txt"))) {
  String line;
  while ((line = reader.readLine()) != null) {
    System.out.println(line);
  }
} catch (IOException e) {
  System.out.println("Error reading file: " + e.getMessage());
}
```

**Reference:** [Oracle Docs - The try-with-resources Statement](https://docs.oracle.com/javase/tutorial/essential/exceptions/tryResourceClose.html)

---

### Invoking Object and Class Methods

#### Invoking Object Methods

Object methods are actions performed on an object.

**Syntax:**

```java
objectReference.methodName(arguments);
```

**Example:**

```java
String message = "hello";
int length = message.length(); // Invokes the length() method on the message object
System.out.println(length); // Output: 5
```

#### Invoking Class Methods

Class methods (static methods) are associated with the class itself rather than an instance of the class.

**Syntax:**

```java
ClassName.methodName(arguments);
```

**Example:**

```java
double pi = Math.PI; // Accessing the static field PI in the Math class
double sqrt = Math.sqrt(16); // Invoking the static method sqrt() in the Math class
```

**Reference:** [Oracle Docs - Classes and Objects](https://docs.oracle.com/javase/tutorial/java/javaOO/index.html)

---

### Accessing Object and Class Fields

#### Accessing Object Fields

Object fields store data associated with an object.

**Syntax:**

```java
objectReference.fieldName;
```

**Example:**

```java
Dog myDog = new Dog("Buddy", 5);
String dogName = myDog.name; // Accessing the name field of the myDog object
```

#### Accessing Class Fields

Class fields (static fields) are associated with the class itself and are shared across all instances of the class.

**Syntax:**

```java
ClassName.fieldName;
```

**Example:**

```java
int maxThreads = Thread.MAX_PRIORITY; // Accessing the static field MAX_PRIORITY in the Thread class
```

**Note:** It's good practice to encapsulate fields and access them through methods (getters and setters).

---

### Iterables and Iterators

**Iterable** and **Iterator** are interfaces in Java that facilitate iterating over elements in a collection.

* **Iterable:** An object that implements the `Iterable` interface allows iteration using a `for-each` loop.
* **Iterator:** The `Iterator` interface provides methods like `hasNext()` to check if there are more elements and `next()` to retrieve the next element.

**Example:**

```java
List<String> names = Arrays.asList("Alice", "Bob", "Charlie");

// Using an Iterator
Iterator<String> iterator = names.iterator();
while (iterator.hasNext()) {
  System.out.println(iterator.next());
}

// Using a for-each loop (implicitly using an Iterator)
for (String name : names) {
  System.out.println(name);
}
```

**Reference:** [Oracle Docs - The Collections Framework](https://docs.oracle.com/javase/tutorial/collections/index.html)

---

This comprehensive guide provided detailed explanations of Java control flow statements, object manipulation, and iterators. Remember to refer to the official Java documentation and other resources for further exploration and in-depth understanding. Happy coding!
